u"""
A `Flask <http://flask.pocoo.org/>`_ server for serving predictions
from a single AllenNLP model. It also includes a very, very bare-bones
web front-end for exploring predictions (or you can provide your own).

For example, if you have your own predictor and model in the `my_stuff` package,
and you want to use the default HTML, you could run this like

```
python -m allennlp.service.server_simple \
    --archive-path allennlp/tests/fixtures/bidaf/serialization/model.tar.gz \
    --predictor machine-comprehension \
    --title "Demo of the Machine Comprehension Text Fixture" \
    --field-name question --field-name passage
```
"""
from __future__ import absolute_import
#typing
import argparse
import json
import logging
import os
from string import Template
import sys

from flask import Flask, request, Response, jsonify, send_file, send_from_directory
from flask_cors import CORS
from gevent.pywsgi import WSGIServer

from allennlp.common import JsonDict
from allennlp.common.util import import_submodules
from allennlp.models.archival import load_archive
from allennlp.predictors import Predictor

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name

class ServerError(Exception):
    status_code = 400

    def __init__(self, message, status_code=None, payload=None):
        Exception.__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code
        self.payload = payload

    def to_dict(self):
        error_dict = dict(self.payload or ())
        error_dict[u'message'] = self.message
        return error_dict


def make_app(predictor           ,
             field_names            = None,
             static_dir      = None,
             sanitizer                                 = None,
             title      = u"AllenNLP Demo")         :
    u"""
    Creates a Flask app that serves up the provided ``Predictor``
    along with a front-end for interacting with it.

    If you want to use the built-in bare-bones HTML, you must provide the
    field names for the inputs (which will be used both as labels
    and as the keys in the JSON that gets sent to the predictor).

    If you would rather create your own HTML, call it index.html
    and provide its directory as ``static_dir``. In that case you
    don't need to supply the field names -- that information should
    be implicit in your demo site. (Probably the easiest thing to do
    is just start with the bare-bones HTML and modify it.)

    In addition, if you want somehow transform the JSON prediction
    (e.g. by removing probabilities or logits)
    you can do that by passing in a ``sanitizer`` function.
    """
    if static_dir is not None:
        static_dir = os.path.abspath(static_dir)
        if not os.path.exists(static_dir):
            logger.error(u"app directory %s does not exist, aborting", static_dir)
            sys.exit(-1)
    elif static_dir is None and field_names is None:
        print u"Neither build_dir nor field_names passed. Demo won't render on this port.\n"
              u"You must use nodejs + react app to interact with the server."

    app = Flask(__name__)  # pylint: disable=invalid-name

    @app.errorhandler(ServerError)
    def handle_invalid_usage(error             )            :  # pylint: disable=unused-variable
        response = jsonify(error.to_dict())
        response.status_code = error.status_code
        return response

    @app.route(u'/')
    def index()            : # pylint: disable=unused-variable
        if static_dir is not None:
            return send_file(os.path.join(static_dir, u'index.html'))
        else:
            html = _html(title, field_names)
            return Response(response=html, status=200)

    @app.route(u'/predict', methods=[u'POST', u'OPTIONS'])
    def predict()            :  # pylint: disable=unused-variable
        u"""make a prediction using the specified model and return the results"""
        if request.method == u"OPTIONS":
            return Response(response=u"", status=200)

        data = request.get_json()

        prediction = predictor.predict_json(data)
        if sanitizer is not None:
            prediction = sanitizer(prediction)

        log_blob = {u"inputs": data, u"outputs": prediction}
        logger.info(u"prediction: %s", json.dumps(log_blob))

        return jsonify(prediction)

    @app.route(u'/<path:path>')
    def static_proxy(path     )            : # pylint: disable=unused-variable
        if static_dir is not None:
            return send_from_directory(static_dir, path)
        else:
            raise ServerError(u"static_dir not specified", 404)

    return app


def main(args):
    # Executing this file with no extra options runs the simple service with the bidaf test fixture
    # and the machine-comprehension predictor. There's no good reason you'd want
    # to do this, except possibly to test changes to the stock HTML).

    parser = argparse.ArgumentParser(description=u'Serve up a simple model')

    parser.add_argument(u'--archive-path', type=unicode, required=True, help=u'path to trained archive file')
    parser.add_argument(u'--predictor', type=unicode, required=True, help=u'name of predictor')
    parser.add_argument(u'--static-dir', type=unicode, help=u'serve index.html from this directory')
    parser.add_argument(u'--title', type=unicode, help=u'change the default page title', default=u"AllenNLP Demo")
    parser.add_argument(u'--field-name', type=unicode, action=u'append',
                        help=u'field names to include in the demo')
    parser.add_argument(u'--port', type=int, default=8000, help=u'port to serve the demo on')

    parser.add_argument(u'--include-package',
                        type=unicode,
                        action=u'append',
                        default=[],
                        help=u'additional packages to include')

    args = parser.parse_args(args)

    # Load modules
    for package_name in args.include_package:
        import_submodules(package_name)

    archive = load_archive(args.archive_path)
    predictor = Predictor.from_archive(archive, args.predictor)
    field_names = args.field_name

    app = make_app(predictor=predictor,
                   field_names=field_names,
                   static_dir=args.static_dir,
                   title=args.title)
    CORS(app)

    http_server = WSGIServer((u'0.0.0.0', args.port), app)
    print "Model loaded, serving demo on port {args.port}"
    http_server.serve_forever()

#
# HTML and Templates for the default bare-bones app are below
#

_PAGE_TEMPLATE = Template(u"""
<html>
    <head>
        <title>
            $title
        </title>
        <style>
            $css
        </style>
    </head>
    <body>
        <div class="pane-container">
            <div class="pane model">
                <div class="pane__left model__input">
                    <div class="model__content">
                        <h2><span>$title</span></h2>
                        <div class="model__content">
                            $inputs
                            <div class="form__field form__field--btn">
                                <button type="button" class="btn btn--icon-disclosure" onclick="predict()">Predict</button>
                            </div>
                        </div>
                    </div>
                </div>
                <div class="pane__right model__output model__output--empty">
                    <div class="pane__thumb"></div>
                    <div class="model__content">
                        <div id="output" class="output">
                            <div class="placeholder">
                                <div class="placeholder__content">
                                    <p>Run model to view results</p>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </body>
    <script>
    function predict() {
        var quotedFieldList = $qfl;
        var data = {};
        quotedFieldList.forEach(function(fieldName) {
            data[fieldName] = document.getElementById("input-" + fieldName).value;
        })

        var xhr = new XMLHttpRequest();
        xhr.open('POST', '/predict');
        xhr.setRequestHeader('Content-Type', 'application/json');
        xhr.onload = function() {
            if (xhr.status == 200) {
                // If you want a more impressive visualization than just
                // outputting the raw JSON, change this part of the code.
                var htmlResults = "<pre>" + JSON.stringify(JSON.parse(xhr.responseText), null, 2) + "</pre>";

                document.getElementById("output").innerHTML = htmlResults;
            }
        };
        xhr.send(JSON.stringify(data));
    }
    </script>
</html>
""")

_SINGLE_INPUT_TEMPLATE = Template(u"""
        <div class="form__field">
            <label for="input-$field_name">$field_name</label>
            <input type="text" id="input-$field_name" type="text" required value placeholder="input goes here">
        </div>
""")


_CSS = u"""
body,
html {
  min-width: 48em;
  background: #f9fafc;
  font-size: 16px
}

* {
  font-family: sans-serif;
  color: #232323
}

section {
  background: #fff
}

code,
code span,
pre,
.output {
  font-family: 'Roboto Mono', monospace!important
}

code {
  background: #f6f8fa
}

li,
p,
td,
th {
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  font-size: 1.125em;
  line-height: 1.5em;
  margin: 1.2em 0
}

pre {
  margin: 2em 0
}

h1,
h2 {
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  font-weight: 300
}

h2 {
  font-size: 2em;
  color: rgba(35, 35, 35, .75)
}

img {
  max-width: 100%
}

hr {
  display: block;
  border: none;
  height: .375em;
  background: #f6f8fa
}

blockquote,
hr {
  margin: 2.4em 0
}

.btn {
  text-decoration: none;
  cursor: pointer;
  text-transform: uppercase;
  font-size: 1em;
  margin: 0;
  -moz-appearance: none;
  -webkit-appearance: none;
  border: none;
  color: #fff!important;
  display: block;
  background: #2085bc;
  padding: .9375em 3.625em;
  -webkit-transition: background-color .2s ease, opacity .2s ease;
  transition: background-color .2s ease, opacity .2s ease
}

.btn.btn--blue {
  background: #2085bc
}

.btn:focus,
.btn:hover {
  background: #40affd;
  outline: 0
}

.btn:focus {
  box-shadow: 0 0 1.25em rgba(50, 50, 150, .05)
}

.btn:active {
  opacity: .66;
  background: #2085bc;
  -webkit-transition-duration: 0s;
  transition-duration: 0s
}

.btn:disabled,
.btn:disabled:active,
.btn:disabled:hover {
  cursor: default;
  background: #d0dae3
}

form {
  display: block
}

.form__field {
  -webkit-transition: margin .2s ease;
  transition: margin .2s ease
}

.form__field+.form__field {
  margin-top: 2.5em
}

.form__field label {
  display: block;
  font-weight: 600;
  font-size: 1.125em
}

.form__field label+* {
  margin-top: 1.25em
}

.form__field input[type=text],
.form__field textarea {
  -moz-appearance: none;
  -webkit-appearance: none;
  width: 100%;
  font-size: 1em;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  padding: .8125em 1.125em;
  color: #232323;
  border: .125em solid #d4dce2;
  display: block;
  box-sizing: border-box;
  -webkit-transition: background-color .2s ease, color .2s ease, border-color .2s ease, opacity .2s ease;
  transition: background-color .2s ease, color .2s ease, border-color .2s ease, opacity .2s ease
}

.form__field input[type=text]::-webkit-input-placeholder,
.form__field textarea::-webkit-input-placeholder {
  color: #b4b4b4
}

.form__field input[type=text]:-moz-placeholder,
.form__field textarea:-moz-placeholder {
  color: #b4b4b4
}

.form__field input[type=text]::-moz-placeholder,
.form__field textarea::-moz-placeholder {
  color: #b4b4b4
}

.form__field input[type=text]:-ms-input-placeholder,
.form__field textarea:-ms-input-placeholder {
  color: #b4b4b4
}

.form__field input[type=text]:focus,
.form__field textarea:focus {
  outline: 0;
  border-color: #63a7d4;
  box-shadow: 0 0 1.25em rgba(50, 50, 150, .05)
}

.form__field textarea {
  resize: vertical;
  min-height: 8.25em
}

.form__field .btn {
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  -webkit-touch-callout: none
}

.form__field--btn {
  display: -webkit-box;
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex;
  -webkit-flex-direction: row;
  -ms-flex-direction: row;
  -webkit-box-orient: horizontal;
  -webkit-box-direction: normal;
  flex-direction: row;
  -webkit-justify-content: flex-end;
  -ms-justify-content: flex-end;
  -webkit-box-pack: end;
  -ms-flex-pack: end;
  justify-content: flex-end
}

@media screen and (max-height:760px) {
  .form__instructions {
    margin: 1.875em 0 1.125em
  }
  .form__field:not(.form__field--btn)+.form__field:not(.form__field--btn) {
    margin-top: 1.25em
  }
}

body,
html {
  width: 100%;
  height: 100%;
  margin: 0;
  padding: 0;
  font-family: 'Source Sans Pro', sans-serif
}

h1 {
  font-weight: 300
}

.model__output {
  background: #fff
}

.model__output.model__output--empty {
  background: 0 0
}

.placeholder {
  width: 100%;
  height: 100%;
  display: -webkit-box;
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex;
  -webkit-align-items: center;
  -ms-flex-align: center;
  -webkit-box-align: center;
  align-items: center;
  -webkit-justify-content: center;
  -ms-justify-content: center;
  -webkit-box-pack: center;
  -ms-flex-pack: center;
  justify-content: center;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  -webkit-touch-callout: none;
  cursor: default
}

.placeholder .placeholder__content {
  display: -webkit-box;
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex;
  -webkit-flex-direction: column;
  -ms-flex-direction: column;
  -webkit-box-orient: vertical;
  -webkit-box-direction: normal;
  flex-direction: column;
  -webkit-align-items: center;
  -ms-flex-align: center;
  -webkit-box-align: center;
  align-items: center;
  text-align: center
}

.placeholder svg {
  display: block
}

.placeholder svg.placeholder__empty,
.placeholder svg.placeholder__error {
  width: 6em;
  height: 3.625em;
  fill: #e1e5ea;
  margin-bottom: 2em
}

.placeholder svg.placeholder__error {
  width: 4.4375em;
  height: 4em
}

.placeholder p {
  font-size: 1em;
  margin: 0;
  padding: 0;
  color: #9aa8b2
}

.placeholder svg.placeholder__working {
  width: 3.4375em;
  height: 3.4375em;
  -webkit-animation: working 1s infinite linear;
  animation: working 1s infinite linear
}

@-webkit-keyframes working {
  0% {
    -webkit-transform: rotate(0deg)
  }
  100% {
    -webkit-transform: rotate(360deg)
  }
}

@keyframes working {
  0% {
    -webkit-transform: rotate(0deg);
    -ms-transform: rotate(0deg);
    transform: rotate(0deg)
  }
  100% {
    -webkit-transform: rotate(360deg);
    -ms-transform: rotate(360deg);
    transform: rotate(360deg)
  }
}

.model__content {
  padding: 1.875em 2.5em;
  margin: auto;
  -webkit-transition: padding .2s ease;
  transition: padding .2s ease
}

.model__content:not(.model__content--srl-output) {
  max-width: 61.25em
}

.model__content h2 {
  margin: 0;
  padding: 0;
  font-size: 1em
}

.model__content h2 span {
  font-size: 2em;
  color: rgba(35, 35, 35, .75)
}

.model__content h2 .tooltip,
.model__content h2 span {
  vertical-align: top
}

.model__content h2 span+.tooltip {
  margin-left: .4375em
}

.model__content>h2:first-child {
  margin: -.25em 0 0 -.03125em
}

.model__content__summary {
  font-size: 1em;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  padding: 1.25em;
  background: #f6f8fa
}

@media screen and (min-height:800px) {
  .model__content {
    padding-top: 4.6vh;
    padding-bottom: 4.6vh
  }
}

.pane-container {
  display: -webkit-box;
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex;
  -webkit-flex-direction: column;
  -ms-flex-direction: column;
  -webkit-box-orient: vertical;
  -webkit-box-direction: normal;
  flex-direction: column;
  height: 100%
}

.pane {
  display: -webkit-box;
  display: -ms-flexbox;
  display: -webkit-flex;
  display: flex;
  -webkit-flex-direction: row;
  -ms-flex-direction: row;
  -webkit-box-orient: horizontal;
  -webkit-box-direction: normal;
  flex-direction: row;
  position: relative;
  -webkit-box-flex: 2;
  -webkit-flex: 2;
  -ms-flex: 2;
  flex: 2;
  height: auto;
  min-height: 100%;
  min-height: 34.375em
}

.pane__left,
.pane__right {
  width: 100%;
  height: 100%;
  -webkit-align-self: stretch;
  -ms-flex-item-align: stretch;
  align-self: stretch;
  min-width: 24em;
  min-height: 34.375em
}

.pane__left {
  height: auto;
  min-height: 100%
}

.pane__right {
  width: 100%;
  overflow: auto;
  height: auto;
  min-height: 100%
}

.pane__right .model__content.model__content--srl-output {
  display: inline-block;
  margin: auto
}

.pane__thumb {
  height: auto;
  min-height: 100%;
  margin-left: -.625em;
  position: absolute;
  width: 1.25em
}

.pane__thumb:after {
  display: block;
  position: absolute;
  height: 100%;
  top: 0;
  content: "";
  width: .25em;
  background: #e1e5ea;
  left: .5em
}
"""

def _html(title     , field_names           )       :
    u"""
    Returns bare bones HTML for serving up an input form with the
    specified fields that can render predictions from the configured model.
    """
    inputs = u''.join(_SINGLE_INPUT_TEMPLATE.substitute(field_name=field_name)
                     for field_name in field_names)

    quoted_field_names = ["'{field_name}'" for field_name in field_names]
    quoted_field_list = "[{','.join(quoted_field_names)}]"

    return _PAGE_TEMPLATE.substitute(title=title,
                                     css=_CSS,
                                     inputs=inputs,
                                     qfl=quoted_field_list)

if __name__ == u"__main__":
    main(sys.argv[1:])



# pylint: disable=invalid-name,no-self-use


from __future__ import with_statement
from __future__ import absolute_import
import pytest
import numpy
import torch
import torch.nn.init
from torch.nn.modules.rnn import LSTM
from torch.nn.utils.rnn import pad_packed_sequence, pack_padded_sequence

from allennlp.common.checks import ConfigurationError
from allennlp.common.testing import AllenNlpTestCase
from allennlp.modules.augmented_lstm import AugmentedLstm
from allennlp.nn import InitializerApplicator
from allennlp.nn.util import sort_batch_by_length


class TestAugmentedLSTM(AllenNlpTestCase):
    def setUp(self):
        super(TestAugmentedLSTM, self).setUp()
        tensor = torch.rand([5, 7, 10])
        tensor[0, 3:, :] = 0
        tensor[1, 4:, :] = 0
        tensor[2, 2:, :] = 0
        tensor[3, 6:, :] = 0
        sequence_lengths = torch.LongTensor([3, 4, 2, 6, 7])
        self.random_tensor = tensor
        self.sequence_lengths = sequence_lengths

    def test_variable_length_sequences_return_correctly_padded_outputs(self):
        sorted_tensor, sorted_sequence, _, _ = sort_batch_by_length(self.random_tensor, self.sequence_lengths)
        tensor = pack_padded_sequence(sorted_tensor, sorted_sequence.data.tolist(), batch_first=True)
        lstm = AugmentedLstm(10, 11)
        output, _ = lstm(tensor)
        output_sequence, _ = pad_packed_sequence(output, batch_first=True)

        numpy.testing.assert_array_equal(output_sequence.data[1, 6:, :].numpy(), 0.0)
        numpy.testing.assert_array_equal(output_sequence.data[2, 4:, :].numpy(), 0.0)
        numpy.testing.assert_array_equal(output_sequence.data[3, 3:, :].numpy(), 0.0)
        numpy.testing.assert_array_equal(output_sequence.data[4, 2:, :].numpy(), 0.0)

    def test_variable_length_sequences_run_backward_return_correctly_padded_outputs(self):
        sorted_tensor, sorted_sequence, _, _ = sort_batch_by_length(self.random_tensor, self.sequence_lengths)
        tensor = pack_padded_sequence(sorted_tensor, sorted_sequence.data.tolist(), batch_first=True)
        lstm = AugmentedLstm(10, 11, go_forward=False)
        output, _ = lstm(tensor)
        output_sequence, _ = pad_packed_sequence(output, batch_first=True)

        numpy.testing.assert_array_equal(output_sequence.data[1, 6:, :].numpy(), 0.0)
        numpy.testing.assert_array_equal(output_sequence.data[2, 4:, :].numpy(), 0.0)
        numpy.testing.assert_array_equal(output_sequence.data[3, 3:, :].numpy(), 0.0)
        numpy.testing.assert_array_equal(output_sequence.data[4, 2:, :].numpy(), 0.0)

    def test_augmented_lstm_computes_same_function_as_pytorch_lstm(self):
        augmented_lstm = AugmentedLstm(10, 11)
        pytorch_lstm = LSTM(10, 11, num_layers=1, batch_first=True)
        # Initialize all weights to be == 1.
        initializer = InitializerApplicator([(u".*", lambda tensor: torch.nn.init.constant_(tensor, 1.))])
        initializer(augmented_lstm)
        initializer(pytorch_lstm)

        initial_state = torch.zeros([1, 5, 11])
        initial_memory = torch.zeros([1, 5, 11])

        # Use bigger numbers to avoid floating point instability.
        sorted_tensor, sorted_sequence, _, _ = sort_batch_by_length(self.random_tensor * 5., self.sequence_lengths)
        lstm_input = pack_padded_sequence(sorted_tensor, sorted_sequence.data.tolist(), batch_first=True)

        augmented_output, augmented_state = augmented_lstm(lstm_input, (initial_state, initial_memory))
        pytorch_output, pytorch_state = pytorch_lstm(lstm_input, (initial_state, initial_memory))
        pytorch_output_sequence, _ = pad_packed_sequence(pytorch_output, batch_first=True)
        augmented_output_sequence, _ = pad_packed_sequence(augmented_output, batch_first=True)

        numpy.testing.assert_array_almost_equal(pytorch_output_sequence.data.numpy(),
                                                augmented_output_sequence.data.numpy(), decimal=4)
        numpy.testing.assert_array_almost_equal(pytorch_state[0].data.numpy(),
                                                augmented_state[0].data.numpy(), decimal=4)
        numpy.testing.assert_array_almost_equal(pytorch_state[1].data.numpy(),
                                                augmented_state[1].data.numpy(), decimal=4)

    def test_augmented_lstm_works_with_highway_connections(self):
        augmented_lstm = AugmentedLstm(10, 11, use_highway=True)
        sorted_tensor, sorted_sequence, _, _ = sort_batch_by_length(self.random_tensor, self.sequence_lengths)
        lstm_input = pack_padded_sequence(sorted_tensor, sorted_sequence.data.tolist(), batch_first=True)
        augmented_lstm(lstm_input)

    def test_augmented_lstm_throws_error_on_non_packed_sequence_input(self):
        lstm = AugmentedLstm(3, 5)
        tensor = torch.rand([5, 7, 9])
        with pytest.raises(ConfigurationError):
            lstm(tensor)

    def test_augmented_lstm_is_initialized_with_correct_biases(self):
        lstm = AugmentedLstm(2, 3)
        true_state_bias = numpy.array([0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0])
        numpy.testing.assert_array_equal(lstm.state_linearity.bias.data.numpy(), true_state_bias)

        # Non-highway case.
        lstm = AugmentedLstm(2, 3, use_highway=False)
        true_state_bias = numpy.array([0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0])
        numpy.testing.assert_array_equal(lstm.state_linearity.bias.data.numpy(), true_state_bias)




from __future__ import absolute_import
#typing

#overrides
import torch

from allennlp.data import Vocabulary
from allennlp.data.fields.production_rule_field import ProductionRuleArray
from allennlp.models.model import Model
from allennlp.models.semantic_parsing.wikitables.wikitables_decoder_step import WikiTablesDecoderStep
from allennlp.models.semantic_parsing.wikitables.wikitables_semantic_parser import WikiTablesSemanticParser
from allennlp.modules import Attention, FeedForward, Seq2SeqEncoder, Seq2VecEncoder, TextFieldEmbedder
from allennlp.nn.decoding import BeamSearch
from allennlp.nn.decoding.decoder_trainers import MaximumMarginalLikelihood
from allennlp.semparse.worlds import WikiTablesWorld
from allennlp.semparse import ParsingError


class WikiTablesMmlSemanticParser(WikiTablesSemanticParser):
    u"""
    A ``WikiTablesMmlSemanticParser`` is a :class:`WikiTablesSemanticParser` which is trained to
    maximize the marginal likelihood of an approximate set of logical forms which give the correct
    denotation. This is a re-implementation of the model used for the paper `Neural Semantic Parsing with Type
    Constraints for Semi-Structured Tables
    <https://www.semanticscholar.org/paper/Neural-Semantic-Parsing-with-Type-Constraints-for-Krishnamurthy-Dasigi/8c6f58ed0ebf379858c0bbe02c53ee51b3eb398a>`_,
    by Jayant Krishnamurthy, Pradeep Dasigi, and Matt Gardner (EMNLP 2017).

    WORK STILL IN PROGRESS.  We'll iteratively improve it until we've reproduced the performance of
    the original parser.

    Parameters
    ----------
    vocab : ``Vocabulary``
    question_embedder : ``TextFieldEmbedder``
        Embedder for questions. Passed to super class.
    action_embedding_dim : ``int``
        Dimension to use for action embeddings. Passed to super class.
    encoder : ``Seq2SeqEncoder``
        The encoder to use for the input question. Passed to super class.
    entity_encoder : ``Seq2VecEncoder``
        The encoder to used for averaging the words of an entity. Passed to super class.
    decoder_beam_search : ``BeamSearch``
        When we're not training, this is how we will do decoding.
    max_decoding_steps : ``int``
        When we're decoding with a beam search, what's the maximum number of steps we should take?
        This only applies at evaluation time, not during training. Passed to super class.
    attention : ``Attention``
        We compute an attention over the input question at each step of the decoder, using the
        decoder hidden state as the query.  Passed to WikiTablesDecoderStep.
    training_beam_size : ``int``, optional (default=None)
        If given, we will use a constrained beam search of this size during training, so that we
        use only the top ``training_beam_size`` action sequences according to the model in the MML
        computation.  If this is ``None``, we will use all of the provided action sequences in the
        MML computation.
    use_neighbor_similarity_for_linking : ``bool``, optional (default=False)
        If ``True``, we will compute a max similarity between a question token and the `neighbors`
        of an entity as a component of the linking scores.  This is meant to capture the same kind
        of information as the ``related_column`` feature. Passed to super class.
    dropout : ``float``, optional (default=0)
        If greater than 0, we will apply dropout with this probability after all encoders (pytorch
        LSTMs do not apply dropout to their last layer). Passed to super class.
    num_linking_features : ``int``, optional (default=10)
        We need to construct a parameter vector for the linking features, so we need to know how
        many there are.  The default of 10 here matches the default in the ``KnowledgeGraphField``,
        which is to use all ten defined features. If this is 0, another term will be added to the
        linking score. This term contains the maximum similarity value from the entity's neighbors
        and the question. Passed to super class.
    rule_namespace : ``str``, optional (default=rule_labels)
        The vocabulary namespace to use for production rules.  The default corresponds to the
        default used in the dataset reader, so you likely don't need to modify this. Passed to super
        class.
    tables_directory : ``str``, optional (default=/wikitables/)
        The directory to find tables when evaluating logical forms.  We rely on a call to SEMPRE to
        evaluate logical forms, and SEMPRE needs to read the table from disk itself.  This tells
        SEMPRE where to find the tables. Passed to super class.
    """
    def __init__(self,
                 vocab            ,
                 question_embedder                   ,
                 action_embedding_dim     ,
                 encoder                ,
                 entity_encoder                ,
                 decoder_beam_search            ,
                 max_decoding_steps     ,
                 attention           ,
                 mixture_feedforward              = None,
                 training_beam_size      = None,
                 use_neighbor_similarity_for_linking       = False,
                 dropout        = 0.0,
                 num_linking_features      = 10,
                 rule_namespace      = u'rule_labels',
                 tables_directory      = u'/wikitables/')        :
        use_similarity = use_neighbor_similarity_for_linking
        super(WikiTablesMmlSemanticParser, self).__init__(vocab=vocab,
                         question_embedder=question_embedder,
                         action_embedding_dim=action_embedding_dim,
                         encoder=encoder,
                         entity_encoder=entity_encoder,
                         max_decoding_steps=max_decoding_steps,
                         use_neighbor_similarity_for_linking=use_similarity,
                         dropout=dropout,
                         num_linking_features=num_linking_features,
                         rule_namespace=rule_namespace,
                         tables_directory=tables_directory)
        self._beam_search = decoder_beam_search
        self._decoder_trainer = MaximumMarginalLikelihood(training_beam_size)
        self._decoder_step = WikiTablesDecoderStep(encoder_output_dim=self._encoder.get_output_dim(),
                                                   action_embedding_dim=action_embedding_dim,
                                                   input_attention=attention,
                                                   num_start_types=self._num_start_types,
                                                   num_entity_types=self._num_entity_types,
                                                   mixture_feedforward=mixture_feedforward,
                                                   dropout=dropout)

    #overrides
    def forward(self,  # type: ignore
                question                             ,
                table                             ,
                world                       ,
                actions                                 ,
                example_lisp_string            = None,
                target_action_sequences                   = None,
                metadata                       = None)                           :
        # pylint: disable=arguments-differ
        u"""
        In this method we encode the table entities, link them to words in the question, then
        encode the question. Then we set up the initial state for the decoder, and pass that
        state off to either a DecoderTrainer, if we're training, or a BeamSearch for inference,
        if we're not.

        Parameters
        ----------
        question : Dict[str, torch.LongTensor]
           The output of ``TextField.as_array()`` applied on the question ``TextField``. This will
           be passed through a ``TextFieldEmbedder`` and then through an encoder.
        table : ``Dict[str, torch.LongTensor]``
            The output of ``KnowledgeGraphField.as_array()`` applied on the table
            ``KnowledgeGraphField``.  This output is similar to a ``TextField`` output, where each
            entity in the table is treated as a "token", and we will use a ``TextFieldEmbedder`` to
            get embeddings for each entity.
        world : ``List[WikiTablesWorld]``
            We use a ``MetadataField`` to get the ``World`` for each input instance.  Because of
            how ``MetadataField`` works, this gets passed to us as a ``List[WikiTablesWorld]``,
        actions : ``List[List[ProductionRuleArray]]``
            A list of all possible actions for each ``World`` in the batch, indexed into a
            ``ProductionRuleArray`` using a ``ProductionRuleField``.  We will embed all of these
            and use the embeddings to determine which action to take at each timestep in the
            decoder.
        example_lisp_string : ``List[str]``, optional (default = None)
            The example (lisp-formatted) string corresponding to the given input.  This comes
            directly from the ``.examples`` file provided with the dataset.  We pass this to SEMPRE
            when evaluating denotation accuracy; it is otherwise unused.
        target_action_sequences : torch.Tensor, optional (default = None)
           A list of possibly valid action sequences, where each action is an index into the list
           of possible actions.  This tensor has shape ``(batch_size, num_action_sequences,
           sequence_length)``.
        metadata : ``List[Dict[str, Any]]``, optional, (default = None)
            Metadata containing the original tokenized question within a 'question_tokens' key.
        """
        initial_info = self._get_initial_state_and_scores(question, table, world, actions)
        initial_state = initial_info[u"initial_state"]
        linking_scores = initial_info[u"linking_scores"]
        feature_scores = initial_info[u"feature_scores"]
        similarity_scores = initial_info[u"similarity_scores"]
        batch_size = list(question.values())[0].size(0)
        if target_action_sequences is not None:
            # Remove the trailing dimension (from ListField[ListField[IndexField]]).
            target_action_sequences = target_action_sequences.squeeze(-1)
            target_mask = target_action_sequences != self._action_padding_index
        else:
            target_mask = None

        if self.training:
            return self._decoder_trainer.decode(initial_state,
                                                self._decoder_step,
                                                (target_action_sequences, target_mask))
        else:
            # TODO(pradeep): Most of the functionality in this black can be moved to the super
            # class.
            action_mapping = {}
            for batch_index, batch_actions in enumerate(actions):
                for action_index, action in enumerate(batch_actions):
                    action_mapping[(batch_index, action_index)] = action[0]
            outputs                 = {u'action_mapping': action_mapping}
            if target_action_sequences is not None:
                outputs[u'loss'] = self._decoder_trainer.decode(initial_state,
                                                               self._decoder_step,
                                                               (target_action_sequences, target_mask))[u'loss']
            num_steps = self._max_decoding_steps
            # This tells the state to start keeping track of debug info, which we'll pass along in
            # our output dictionary.
            initial_state.debug_info = [[] for _ in range(batch_size)]
            best_final_states = self._beam_search.search(num_steps,
                                                         initial_state,
                                                         self._decoder_step,
                                                         keep_final_unfinished_states=False)
            outputs[u'best_action_sequence'] = []
            outputs[u'debug_info'] = []
            outputs[u'entities'] = []
            outputs[u'linking_scores'] = linking_scores
            if feature_scores is not None:
                outputs[u'feature_scores'] = feature_scores
            outputs[u'similarity_scores'] = similarity_scores
            outputs[u'logical_form'] = []
            for i in range(batch_size):
                # Decoding may not have terminated with any completed logical forms, if `num_steps`
                # isn't long enough (or if the model is not trained enough and gets into an
                # infinite action loop).
                if i in best_final_states:
                    best_action_indices = best_final_states[i][0].action_history[0]
                    if target_action_sequences is not None:
                        # Use a Tensor, not a Variable, to avoid a memory leak.
                        targets = target_action_sequences[i].data
                        sequence_in_targets = 0
                        sequence_in_targets = self._action_history_match(best_action_indices, targets)
                        self._action_sequence_accuracy(sequence_in_targets)
                    action_strings = [action_mapping[(i, action_index)] for action_index in best_action_indices]
                    try:
                        logical_form = world[i].get_logical_form(action_strings, add_var_function=False)
                        self._has_logical_form(1.0)
                    except ParsingError:
                        self._has_logical_form(0.0)
                        logical_form = u'Error producing logical form'
                    if example_lisp_string:
                        self._denotation_accuracy(logical_form, example_lisp_string[i])
                    outputs[u'best_action_sequence'].append(action_strings)
                    outputs[u'logical_form'].append(logical_form)
                    outputs[u'debug_info'].append(best_final_states[i][0].debug_info[0])  # type: ignore
                    outputs[u'entities'].append(world[i].table_graph.entities)
                else:
                    outputs[u'logical_form'].append(u'')
                    self._has_logical_form(0.0)
                    if example_lisp_string:
                        self._denotation_accuracy(None, example_lisp_string[i])
            if metadata is not None:
                outputs[u"question_tokens"] = [x[u"question_tokens"] for x in metadata]
                outputs[u"original_table"] = [x[u"original_table"] for x in metadata]
            return outputs

WikiTablesMmlSemanticParser = Model.register(u"wikitables_mml_parser")(WikiTablesMmlSemanticParser)





from __future__ import division
from __future__ import absolute_import
import atexit
import logging
import os
import pathlib
import subprocess

#overrides

from allennlp.common.file_utils import cached_path
from allennlp.training.metrics.metric import Metric

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name

SEMPRE_EXECUTOR_JAR = u"https://s3-us-west-2.amazonaws.com/allennlp/misc/wikitables-executor-0.1.0.jar"
ABBREVIATIONS_FILE = u"https://s3-us-west-2.amazonaws.com/allennlp/misc/wikitables-abbreviations.tsv"
GROW_FILE = u"https://s3-us-west-2.amazonaws.com/allennlp/misc/wikitables-grow.grammar"
SEMPRE_DIR = unicode(pathlib.Path(u'data/'))
SEMPRE_ABBREVIATIONS_PATH = os.path.join(SEMPRE_DIR, u"abbreviations.tsv")
SEMPRE_GRAMMAR_PATH = os.path.join(SEMPRE_DIR, u"grow.grammar")


class WikiTablesAccuracy(Metric):
    def __init__(self, table_directory     )        :
        self._table_directory = table_directory
        self._executor_process: subprocess.Popen = None
        self._create_sempre_executor()
        self._count = 0
        self._correct = 0

    #overrides
    def __call__(self, logical_form     , example_lisp_string     ):  # type: ignore
        u"""
        Parameters
        ----------
        example_lisp_string : ``str``
            The value to average.
        """
        denotation_correct = self.evaluate_logical_form(logical_form, example_lisp_string)
        if denotation_correct:
            self._correct += 1
        self._count += 1

    #overrides
    def get_metric(self, reset       = False)         :
        accuracy = self._correct / self._count if self._count > 0 else 0
        if reset:
            self.reset()
        return accuracy

    #overrides
    def reset(self):
        self._count = 0
        self._correct = 0

    def __str__(self):
        return "WikiTablesAccuracy(correct={self._correct}, count={self._count})"

    def evaluate_logical_form(self, logical_form     , example_lisp_string     )        :
        if not logical_form or logical_form.startswith(u'Error'):
            return False
        if example_lisp_string[-1] != u'\n':
            example_lisp_string += u'\n'
        if logical_form[-1] != u'\n':
            logical_form += u'\n'
        self._executor_process.stdin.write(example_lisp_string.encode(u'utf-8'))
        self._executor_process.stdin.write(logical_form.encode(u'utf-8'))
        self._executor_process.stdin.flush()
        result = self._executor_process.stdout.readline().decode().strip()
        return result == u'1.0'

    def _create_sempre_executor(self)        :
        u"""
        Creates a server running SEMPRE that we can send logical forms to for evaluation.  This
        uses inter-process communication, because SEMPRE is java code.  We also need to be careful
        to clean up the process when our program exits.
        """
        if self._executor_process:
            return

        # It'd be much nicer to just use `cached_path` for these files.  However, the SEMPRE jar
        # that we're using expects to find these files in a particular location, so we need to make
        # sure we put the files in that location.
        os.makedirs(SEMPRE_DIR, exist_ok=True)
        abbreviations_path = os.path.join(SEMPRE_DIR, u'abbreviations.tsv')
        if not os.path.exists(abbreviations_path):
            subprocess.run('wget {ABBREVIATIONS_FILE}', shell=True)
            subprocess.run('mv wikitables-abbreviations.tsv {abbreviations_path}', shell=True)

        grammar_path = os.path.join(SEMPRE_DIR, u'grow.grammar')
        if not os.path.exists(grammar_path):
            subprocess.run('wget {GROW_FILE}', shell=True)
            subprocess.run('mv wikitables-grow.grammar {grammar_path}', shell=True)

        args = [u'java', u'-jar', cached_path(SEMPRE_EXECUTOR_JAR), u'serve', self._table_directory]
        self._executor_process = subprocess.Popen(args,
                                                  stdin=subprocess.PIPE,
                                                  stdout=subprocess.PIPE,
                                                  bufsize=1)

        lines = []
        for _ in range(6):
            # SEMPRE outputs six lines of stuff when it loads that I can't disable.  So, we clear
            # that here.
            lines.append(unicode(self._executor_process.stdout.readline()))
        assert u'Parser' in lines[-1], u"SEMPRE server output unexpected; the server may have changed"
        logger.info(u"Started SEMPRE server for evaluating logical forms")

        # This is supposed to ensure that the subprocess gets killed when python exits.
        atexit.register(self._stop_sempre_executor)

    def _stop_sempre_executor(self)        :
        if not self._executor_process:
            return
        self._executor_process.terminate()
        self._executor_process = None
        logger.info(u"Stopped SEMPRE server")



# pylint: disable=unused-import

from __future__ import absolute_import
import warnings

from allennlp.predictors.semantic_role_labeler import SemanticRoleLabelerPredictor
warnings.warn(u"allennlp.service.predictors.* has been deprecated."
              u" Please use allennlp.predictors.*", FutureWarning)



# pylint: disable=no-self-use,invalid-name

from __future__ import absolute_import
import torch
from torch.nn.init import constant_
from allennlp.common.params import Params
from allennlp.nn import InitializerApplicator
from allennlp.nn.regularizers import L1Regularizer, L2Regularizer, RegularizerApplicator
from allennlp.common.testing import AllenNlpTestCase


class TestRegularizers(AllenNlpTestCase):
    def test_l1_regularization(self):
        model = torch.nn.Sequential(
                torch.nn.Linear(5, 10),
                torch.nn.Linear(10, 5)
        )
        initializer = InitializerApplicator([(u".*", lambda tensor: constant_(tensor, -1))])
        initializer(model)
        value = RegularizerApplicator([(u"", L1Regularizer(1.0))])(model)
        # 115 because of biases.
        assert value.data.numpy() == 115.0

    def test_l2_regularization(self):
        model = torch.nn.Sequential(
                torch.nn.Linear(5, 10),
                torch.nn.Linear(10, 5)
        )
        initializer = InitializerApplicator([(u".*", lambda tensor: constant_(tensor, 0.5))])
        initializer(model)
        value = RegularizerApplicator([(u"", L2Regularizer(1.0))])(model)
        assert value.data.numpy() == 28.75

    def test_regularizer_applicator_respects_regex_matching(self):
        model = torch.nn.Sequential(
                torch.nn.Linear(5, 10),
                torch.nn.Linear(10, 5)
        )
        initializer = InitializerApplicator([(u".*", lambda tensor: constant_(tensor, 1.))])
        initializer(model)
        value = RegularizerApplicator([(u"weight", L2Regularizer(0.5)),
                                       (u"bias", L1Regularizer(1.0))])(model)
        assert value.data.numpy() == 65.0

    def test_from_params(self):
        params = Params({u"regularizers": [(u"conv", u"l1"), (u"linear", {u"type": u"l2", u"alpha": 10})]})
        regularizer_applicator = RegularizerApplicator.from_params(params.pop(u"regularizers"))
        regularizers = regularizer_applicator._regularizers  # pylint: disable=protected-access

        conv = linear = None
        for regex, regularizer in regularizers:
            if regex == u"conv":
                conv = regularizer
            elif regex == u"linear":
                linear = regularizer

        assert isinstance(conv, L1Regularizer)
        assert isinstance(linear, L2Regularizer)
        assert linear.alpha == 10




from __future__ import absolute_import
import math

#overrides
import torch

from allennlp.modules.similarity_functions.similarity_function import SimilarityFunction


class DotProductSimilarity(SimilarityFunction):
    u"""
    This similarity function simply computes the dot product between each pair of vectors, with an
    optional scaling to reduce the variance of the output elements.

    Parameters
    ----------
    scale_output : ``bool``, optional
        If ``True``, we will scale the output by ``math.sqrt(tensor.size(-1))``, to reduce the
        variance in the result.
    """
    def __init__(self, scale_output       = False)        :
        super(DotProductSimilarity, self).__init__()
        self._scale_output = scale_output

    #overrides
    def forward(self, tensor_1              , tensor_2              )                :
        result = (tensor_1 * tensor_2).sum(dim=-1)
        if self._scale_output:
            result *= math.sqrt(tensor_1.size(-1))
        return result

DotProductSimilarity = SimilarityFunction.register(u"dot_product")(DotProductSimilarity)





from __future__ import division
from __future__ import absolute_import
#overrides
import torch

from allennlp.modules.similarity_functions.similarity_function import SimilarityFunction


class CosineSimilarity(SimilarityFunction):
    u"""
    This similarity function simply computes the cosine similarity between each pair of vectors.  It has
    no parameters.
    """
    #overrides
    def forward(self, tensor_1              , tensor_2              )                :
        normalized_tensor_1 = tensor_1 / tensor_1.norm(dim=-1, keepdim=True)
        normalized_tensor_2 = tensor_2 / tensor_2.norm(dim=-1, keepdim=True)
        return (normalized_tensor_1 * normalized_tensor_2).sum(dim=-1)

CosineSimilarity = SimilarityFunction.register(u"cosine")(CosineSimilarity)



# pylint: disable=line-too-long

from __future__ import absolute_import
from allennlp.modules.span_extractors.span_extractor import SpanExtractor
from allennlp.modules.span_extractors.endpoint_span_extractor import EndpointSpanExtractor
from allennlp.modules.span_extractors.self_attentive_span_extractor import SelfAttentiveSpanExtractor
from allennlp.modules.span_extractors.bidirectional_endpoint_span_extractor import BidirectionalEndpointSpanExtractor



u"""
A wrapper that unrolls the second (time) dimension of a tensor
into the first (batch) dimension, applies some other ``Module``,
and then rolls the time dimension back up.
"""


from __future__ import absolute_import
import torch


class TimeDistributed(torch.nn.Module):
    u"""
    Given an input shaped like ``(batch_size, time_steps, [rest])`` and a ``Module`` that takes
    inputs like ``(batch_size, [rest])``, ``TimeDistributed`` reshapes the input to be
    ``(batch_size * time_steps, [rest])``, applies the contained ``Module``, then reshapes it back.

    Note that while the above gives shapes with ``batch_size`` first, this ``Module`` also works if
    ``batch_size`` is second - we always just combine the first two dimensions, then split them.
    """
    def __init__(self, module):
        super(TimeDistributed, self).__init__()
        self._module = module

    def forward(self, *inputs):  # pylint: disable=arguments-differ
        reshaped_inputs = []
        for input_tensor in inputs:
            input_size = input_tensor.size()
            if len(input_size) <= 2:
                raise RuntimeError(u"No dimension to distribute: " + unicode(input_size))

            # Squash batch_size and time_steps into a single axis; result has shape
            # (batch_size * time_steps, input_size).
            squashed_shape = [-1] + [x for x in input_size[2:]]
            reshaped_inputs.append(input_tensor.contiguous().view(*squashed_shape))

        reshaped_outputs = self._module(*reshaped_inputs)

        # Now get the output back into the right shape.
        # (batch_size, time_steps, [hidden_size])
        new_shape = [input_size[0], input_size[1]] + [x for x in reshaped_outputs.size()[1:]]
        outputs = reshaped_outputs.contiguous().view(*new_shape)

        return outputs



# pylint: disable=no-self-use,invalid-name,protected-access



from __future__ import division
from __future__ import with_statement
from __future__ import absolute_import
import torch
import pytest
import numpy

from allennlp.common.testing import AllenNlpTestCase
from allennlp.common.checks import ConfigurationError
from allennlp.training.metrics import CategoricalAccuracy

class CategoricalAccuracyTest(AllenNlpTestCase):
    def test_categorical_accuracy(self):
        accuracy = CategoricalAccuracy()
        predictions = torch.Tensor([[0.35, 0.25, 0.1, 0.1, 0.2],
                                    [0.1, 0.6, 0.1, 0.2, 0.0]])
        targets = torch.Tensor([0, 3])
        accuracy(predictions, targets)
        actual_accuracy = accuracy.get_metric()
        assert actual_accuracy == 0.50

    def test_top_k_categorical_accuracy(self):
        accuracy = CategoricalAccuracy(top_k=2)
        predictions = torch.Tensor([[0.35, 0.25, 0.1, 0.1, 0.2],
                                    [0.1, 0.6, 0.1, 0.2, 0.0]])
        targets = torch.Tensor([0, 3])
        accuracy(predictions, targets)
        actual_accuracy = accuracy.get_metric()
        assert actual_accuracy == 1.0

    def test_top_k_categorical_accuracy_accumulates_and_resets_correctly(self):
        accuracy = CategoricalAccuracy(top_k=2)
        predictions = torch.Tensor([[0.35, 0.25, 0.1, 0.1, 0.2],
                                    [0.1, 0.6, 0.1, 0.2, 0.0]])
        targets = torch.Tensor([0, 3])
        accuracy(predictions, targets)
        accuracy(predictions, targets)
        accuracy(predictions, torch.Tensor([4, 4]))
        accuracy(predictions, torch.Tensor([4, 4]))
        actual_accuracy = accuracy.get_metric(reset=True)
        assert actual_accuracy == 0.50
        assert accuracy.correct_count == 0.0
        assert accuracy.total_count == 0.0

    def test_top_k_categorical_accuracy_respects_mask(self):
        accuracy = CategoricalAccuracy(top_k=2)
        predictions = torch.Tensor([[0.35, 0.25, 0.1, 0.1, 0.2],
                                    [0.1, 0.6, 0.1, 0.2, 0.0],
                                    [0.1, 0.2, 0.5, 0.2, 0.0]])
        targets = torch.Tensor([0, 3, 0])
        mask = torch.Tensor([0, 1, 1])
        accuracy(predictions, targets, mask)
        actual_accuracy = accuracy.get_metric()
        assert actual_accuracy == 0.50

    def test_top_k_categorical_accuracy_works_for_sequences(self):
        accuracy = CategoricalAccuracy(top_k=2)
        predictions = torch.Tensor([[[0.35, 0.25, 0.1, 0.1, 0.2],
                                     [0.1, 0.6, 0.1, 0.2, 0.0],
                                     [0.1, 0.6, 0.1, 0.2, 0.0]],
                                    [[0.35, 0.25, 0.1, 0.1, 0.2],
                                     [0.1, 0.6, 0.1, 0.2, 0.0],
                                     [0.1, 0.6, 0.1, 0.2, 0.0]]])
        targets = torch.Tensor([[0, 3, 4],
                                [0, 1, 4]])
        accuracy(predictions, targets)
        actual_accuracy = accuracy.get_metric(reset=True)
        numpy.testing.assert_almost_equal(actual_accuracy, 0.6666666)

        # Test the same thing but with a mask:
        mask = torch.Tensor([[0, 1, 1],
                             [1, 0, 1]])
        accuracy(predictions, targets, mask)
        actual_accuracy = accuracy.get_metric(reset=True)
        numpy.testing.assert_almost_equal(actual_accuracy, 0.50)

    def test_top_k_categorical_accuracy_catches_exceptions(self):
        accuracy = CategoricalAccuracy()
        predictions = torch.rand([5, 7])
        out_of_range_labels = torch.Tensor([10, 3, 4, 0, 1])
        with pytest.raises(ConfigurationError):
            accuracy(predictions, out_of_range_labels)

    def test_tie_break_categorical_accuracy(self):
        accuracy = CategoricalAccuracy(tie_break=True)
        predictions = torch.Tensor([[0.35, 0.25, 0.35, 0.35, 0.35],
                                    [0.1, 0.6, 0.1, 0.2, 0.2],
                                    [0.1, 0.0, 0.1, 0.2, 0.2]])
        # Test without mask:
        targets = torch.Tensor([2, 1, 4])
        accuracy(predictions, targets)
        assert accuracy.get_metric(reset=True) == (0.25 + 1 + 0.5)/3.0

        # # # Test with mask
        mask = torch.Tensor([1, 0, 1])
        targets = torch.Tensor([2, 1, 4])
        accuracy(predictions, targets, mask)
        assert accuracy.get_metric(reset=True) == (0.25 + 0.5)/2.0

        # # Test tie-break with sequence
        predictions = torch.Tensor([[[0.35, 0.25, 0.35, 0.35, 0.35],
                                     [0.1, 0.6, 0.1, 0.2, 0.2],
                                     [0.1, 0.0, 0.1, 0.2, 0.2]],
                                    [[0.35, 0.25, 0.35, 0.35, 0.35],
                                     [0.1, 0.6, 0.1, 0.2, 0.2],
                                     [0.1, 0.0, 0.1, 0.2, 0.2]]])
        targets = torch.Tensor([[0, 1, 3],  # 0.25 + 1 + 0.5
                                [0, 3, 4]]) # 0.25 + 0 + 0.5 = 2.5
        accuracy(predictions, targets)
        actual_accuracy = accuracy.get_metric(reset=True)
        numpy.testing.assert_almost_equal(actual_accuracy, 2.5/6.0)

    def test_top_k_and_tie_break_together_catches_exceptions(self):
        with pytest.raises(ConfigurationError):
            CategoricalAccuracy(top_k=2, tie_break=True)

    def test_incorrect_top_k_catches_exceptions(self):
        with pytest.raises(ConfigurationError):
            CategoricalAccuracy(top_k=0)

    def test_does_not_divide_by_zero_with_no_count(self):
        accuracy = CategoricalAccuracy()
        self.assertAlmostEqual(accuracy.get_metric(), 0.0)



# pylint: disable=no-self-use,invalid-name

from __future__ import absolute_import
import numpy
import torch
from torch.nn.utils.rnn import pack_padded_sequence, pad_packed_sequence

from allennlp.modules.stacked_bidirectional_lstm import StackedBidirectionalLstm
from allennlp.modules.seq2seq_encoders import Seq2SeqEncoder
from allennlp.common.testing import AllenNlpTestCase
from allennlp.common.params import Params


class TestStackedBidirectionalLstm(AllenNlpTestCase):
    def test_stacked_bidirectional_lstm_completes_forward_pass(self):
        input_tensor = torch.rand(4, 5, 3)
        input_tensor[1, 4:, :] = 0.
        input_tensor[2, 2:, :] = 0.
        input_tensor[3, 1:, :] = 0.
        input_tensor = pack_padded_sequence(input_tensor, [5, 4, 2, 1], batch_first=True)
        lstm = StackedBidirectionalLstm(3, 7, 3)
        output, _ = lstm(input_tensor)
        output_sequence, _ = pad_packed_sequence(output, batch_first=True)
        numpy.testing.assert_array_equal(output_sequence.data[1, 4:, :].numpy(), 0.0)
        numpy.testing.assert_array_equal(output_sequence.data[2, 2:, :].numpy(), 0.0)
        numpy.testing.assert_array_equal(output_sequence.data[3, 1:, :].numpy(), 0.0)

    def test_stacked_bidirectional_lstm_can_build_from_params(self):
        params = Params({u"type": u"stacked_bidirectional_lstm",
                         u"input_size": 5,
                         u"hidden_size": 9,
                         u"num_layers": 3})
        encoder = Seq2SeqEncoder.from_params(params)

        assert encoder.get_input_dim() == 5
        assert encoder.get_output_dim() == 18
        assert encoder.is_bidirectional



# pylint: disable=invalid-name


from __future__ import division
from __future__ import absolute_import
import copy

import torch

from allennlp.common import Params
from allennlp.common.testing import AllenNlpTestCase
from allennlp.commands.train import train_model
from allennlp.models.archival import load_archive, archive_model


class ArchivalTest(AllenNlpTestCase):
    def setUp(self):
        super(ArchivalTest, self).setUp()

        self.params = Params({
                u"model": {
                        u"type": u"simple_tagger",
                        u"text_field_embedder": {
                                u"tokens": {
                                        u"type": u"embedding",
                                        u"embedding_dim": 5
                                }
                        },
                        u"encoder": {
                                u"type": u"lstm",
                                u"input_size": 5,
                                u"hidden_size": 7,
                                u"num_layers": 2
                        }
                },
                u"dataset_reader": {u"type": u"sequence_tagging"},
                u"train_data_path": unicode(self.FIXTURES_ROOT / u'data' / u'sequence_tagging.tsv'),
                u"validation_data_path": unicode(self.FIXTURES_ROOT / u'data' / u'sequence_tagging.tsv'),
                u"iterator": {u"type": u"basic", u"batch_size": 2},
                u"trainer": {
                        u"num_epochs": 2,
                        u"optimizer": u"adam",
                }
        })

    def test_archiving(self):
        # copy params, since they'll get consumed during training
        params_copy = copy.deepcopy(self.params.as_dict())

        # `train_model` should create an archive
        serialization_dir = self.TEST_DIR / u'archive_test'
        model = train_model(self.params, serialization_dir=serialization_dir)

        archive_path = serialization_dir / u"model.tar.gz"

        # load from the archive
        archive = load_archive(archive_path)
        model2 = archive.model

        # check that model weights are the same
        keys = set(model.state_dict().keys())
        keys2 = set(model2.state_dict().keys())

        assert keys == keys2

        for key in keys:
            assert torch.equal(model.state_dict()[key], model2.state_dict()[key])

        # check that vocabularies are the same
        vocab = model.vocab
        vocab2 = model2.vocab

        assert vocab._token_to_index == vocab2._token_to_index  # pylint: disable=protected-access
        assert vocab._index_to_token == vocab2._index_to_token  # pylint: disable=protected-access

        # check that params are the same
        params2 = archive.config
        assert params2.as_dict() == params_copy

    def test_extra_files(self):

        serialization_dir = self.TEST_DIR / u'serialization'

        # Train a model
        train_model(self.params, serialization_dir=serialization_dir)

        # Archive model, and also archive the training data
        files_to_archive = {u"train_data_path": unicode(self.FIXTURES_ROOT / u'data' / u'sequence_tagging.tsv')}
        archive_model(serialization_dir=serialization_dir, files_to_archive=files_to_archive)

        archive = load_archive(serialization_dir / u'model.tar.gz')
        params = archive.config

        # The param in the data should have been replaced with a temporary path
        # (which we don't know, but we know what it ends with).
        assert params.get(u'train_data_path').endswith(u'/fta/train_data_path')

        # The validation data path should be the same though.
        assert params.get(u'validation_data_path') == unicode(self.FIXTURES_ROOT / u'data' / u'sequence_tagging.tsv')




from __future__ import absolute_import
import logging
#typing

#overrides

from allennlp.common.util import pad_sequence_to_length
from allennlp.data.vocabulary import Vocabulary
from allennlp.data.tokenizers.token import Token
from allennlp.data.token_indexers.token_indexer import TokenIndexer

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


class DepLabelIndexer(TokenIndexer):
    u"""
    This :class:`TokenIndexer` represents tokens by their syntactic dependency label, as determined
    by the ``dep_`` field on ``Token``.

    Parameters
    ----------
    namespace : ``str``, optional (default=``dep_labels``)
        We will use this namespace in the :class:`Vocabulary` to map strings to indices.
    """
    # pylint: disable=no-self-use
    def __init__(self, namespace      = u'dep_labels')        :
        self.namespace = namespace
        self._logged_errors = set()

    #overrides
    def count_vocab_items(self, token       , counter                           ):
        dep_label = token.dep_
        if not dep_label:
            if token.text not in self._logged_errors:
                logger.warning(u"Token had no dependency label: %s", token.text)
                self._logged_errors.add(token.text)
            dep_label = u'NONE'
        counter[self.namespace][dep_label] += 1

    #overrides
    def tokens_to_indices(self,
                          tokens             ,
                          vocabulary            ,
                          index_name     )                        :
        dep_labels = [token.dep_ or u'NONE' for token in tokens]

        return {index_name: [vocabulary.get_token_index(dep_label, self.namespace) for dep_label in dep_labels]}

    #overrides
    def get_padding_token(self)       :
        return 0

    #overrides
    def get_padding_lengths(self, token     )                  :  # pylint: disable=unused-argument
        return {}

    #overrides
    def pad_token_sequence(self,
                           tokens                      ,
                           desired_num_tokens                ,
                           padding_lengths                )                        :  # pylint: disable=unused-argument
        return dict((key, pad_sequence_to_length(val, desired_num_tokens[key]))
                for key, val in list(tokens.items()))

DepLabelIndexer = TokenIndexer.register(u"dependency_label")(DepLabelIndexer)



# pylint: disable=no-self-use,invalid-name



from __future__ import division
from __future__ import absolute_import
from allennlp.data.dataset_readers import UniversalDependenciesDatasetReader
from allennlp.common.testing import AllenNlpTestCase

class TestUniversalDependenciesDatasetReader(AllenNlpTestCase):
    data_path = AllenNlpTestCase.FIXTURES_ROOT / u"data" / u"dependencies.conllu"

    def test_read_from_file(self):
        reader = UniversalDependenciesDatasetReader()
        instances = list(reader.read(unicode(self.data_path)))

        instance = instances[0]
        fields = instance.fields
        assert [t.text for t in fields[u"words"].tokens] == [u'What', u'if', u'Google',
                                                            u'Morphed', u'Into', u'GoogleOS', u'?']

        assert fields[u"pos_tags"].labels == [u'PRON', u'SCONJ', u'PROPN', u'VERB', u'ADP', u'PROPN', u'PUNCT']
        assert fields[u"head_tags"].labels == [u'root', u'mark', u'nsubj', u'advcl',
                                              u'case', u'obl', u'punct']
        assert fields[u"head_indices"].labels == [0, 4, 4, 1, 6, 4, 4]

        instance = instances[1]
        fields = instance.fields
        assert [t.text for t in fields[u"words"].tokens] == [u'What', u'if', u'Google', u'expanded', u'on',
                                                            u'its', u'search', u'-', u'engine', u'(', u'and',
                                                            u'now', u'e-mail', u')', u'wares', u'into', u'a',
                                                            u'full', u'-', u'fledged', u'operating', u'system', u'?']

        assert fields[u"pos_tags"].labels == [u'PRON', u'SCONJ', u'PROPN', u'VERB', u'ADP', u'PRON', u'NOUN',
                                             u'PUNCT', u'NOUN', u'PUNCT', u'CCONJ', u'ADV', u'NOUN', u'PUNCT', u'NOUN',
                                             u'ADP', u'DET', u'ADV', u'PUNCT', u'ADJ', u'NOUN', u'NOUN', u'PUNCT']
        assert fields[u"head_tags"].labels == [u'root', u'mark', u'nsubj', u'advcl', u'case', u'nmod:poss',
                                              u'compound', u'punct', u'compound', u'punct', u'cc', u'advmod',
                                              u'conj', u'punct', u'obl', u'case', u'det', u'advmod', u'punct',
                                              u'amod', u'compound', u'obl', u'punct']
        assert fields[u"head_indices"].labels == [0, 4, 4, 1, 15, 15, 9, 9, 15, 9, 13, 13,
                                                 9, 15, 4, 22, 22, 20, 20, 22, 22, 4, 4]

        instance = instances[2]
        fields = instance.fields
        assert [t.text for t in fields[u"words"].tokens] == [u'[', u'via', u'Microsoft', u'Watch',
                                                            u'from', u'Mary', u'Jo', u'Foley', u']']
        assert fields[u"pos_tags"].labels == [u'PUNCT', u'ADP', u'PROPN', u'PROPN', u'ADP',
                                             u'PROPN', u'PROPN', u'PROPN', u'PUNCT']
        assert fields[u"head_tags"].labels == [u'punct', u'case', u'compound', u'root', u'case',
                                              u'nmod', u'flat', u'flat', u'punct']
        assert fields[u"head_indices"].labels == [4, 4, 4, 0, 6, 4, 6, 6, 4]

        # This instance tests specifically for filtering of elipsis:
        # http://universaldependencies.org/u/overview/specific-syntax.html#ellipsis
        # The original sentence is:
        # "Over 300 Iraqis are reported dead and 500 [reported] wounded in Fallujah alone."
        # But the second "reported" is elided, and as such isn't included in the syntax tree.
        instance = instances[3]
        fields = instance.fields
        assert [t.text for t in fields[u"words"].tokens] == [u'Over', u'300', u'Iraqis', u'are',
                                                            u'reported', u'dead', u'and', u'500', u'wounded',
                                                            u'in', u'Fallujah', u'alone', u'.']
        assert fields[u"pos_tags"].labels == [u'ADV', u'NUM', u'PROPN', u'AUX', u'VERB', u'ADJ',
                                             u'CCONJ', u'NUM', u'ADJ', u'ADP', u'PROPN', u'ADV', u'PUNCT']
        assert fields[u"head_tags"].labels == [u'advmod', u'nummod', u'nsubj:pass', u'aux:pass',
                                              u'root', u'xcomp', u'cc', u'conj', u'orphan', u'case', u'obl',
                                              u'advmod', u'punct']
        assert fields[u"head_indices"].labels == [2, 3, 5, 5, 0, 5, 8, 5, 8, 11, 5, 11, 5]



#!/usr/bin/python

# See the accompanying LICENSE file.

"""This file runs the test suite against several versions of SQLite
and Python to make sure everything is ok in the various combinations.
It only runs on a UNIX like environment.

All the work is done in parallel rather than serially.  This allows
for it to finish a lot sooner.

"""

import os
import sys
import threading
import Queue
import optparse
import traceback

# disable testfileprefix
os.putenv("APSWTESTPREFIX", "")
try:
    del os.environ["APSWTESTPREFIX"]
except KeyError:
    pass

# make sure all extensions are tested
for e in "fts3", "rtree", "icu":
    os.putenv("APSW_TEST_"+e.upper(), e)

def run(cmd):
    status=os.system(cmd)
    if os.WIFEXITED(status):
        code=os.WEXITSTATUS(status)
        if code==0:
            return
        raise Exception("Exited with code "+`code`+": "+cmd)
    raise Exception("Failed with signal "+`os.WTERMSIG(status)`+": "+cmd)

def dotest(pyver, logdir, pybin, pylib, workdir, sqlitever):
    run("set -e ; cd %s ; ( env LD_LIBRARY_PATH=%s APSW_FORCE_DISTUTILS=t %s setup.py fetch --version=%s --all build_test_extension build_ext --inplace --force --enable-all-extensions test -v ) >%s 2>&1" % (workdir, pylib, pybin, sqlitever, os.path.abspath(os.path.join(logdir, "buildruntests.txt"))))

def runtest(workdir, pyver, ucs, sqlitever, logdir):
    pybin, pylib=buildpython(workdir, pyver, ucs, os.path.abspath(os.path.join(logdir, "pybuild.txt")))
    dotest(pyver, logdir, pybin, pylib, workdir, sqlitever)

def threadrun(queue):
    while True:
        d=queue.get()
        if d is None:
            return
        try:
            runtest(**d)
            sys.stdout.write(".")
            sys.stdout.flush()
        except:
            # uncomment to debug problems with this script
            # traceback.print_exc()
            print "\nFAILED", d

def main(PYVERS, UCSTEST, SQLITEVERS, concurrency):
    try:
        del os.environ["APSWTESTPREFIX"]
    except KeyError:
        pass
    print "Test starting"
    os.system("rm -rf apsw.so megatestresults 2>/dev/null ; mkdir megatestresults")
    print "  ... removing old work directory"
    workdir=os.path.abspath("work")
    os.system("rm -rf %s/* 2>/dev/null ; mkdir -p %s" % (workdir, workdir))
    os.system("rm -f src/shell.c") # autogenerated
    os.system('rm -rf $HOME/.local/lib/python*/site-packages/apsw* 2>/dev/null')
    print "      done"

    queue=Queue.Queue()
    threads=[]

    for pyver in PYVERS:
        for ucs in UCSTEST:
            if pyver=="system" or pyver>="3.3":
                if ucs!=2: continue
                ucs=0
            for sqlitever in SQLITEVERS:
                print "Python",pyver,"ucs",ucs,"   SQLite",sqlitever
                workdir=os.path.abspath(os.path.join("work", "py%s-ucs%d-sq%s" % (pyver, ucs, sqlitever)))
                logdir=os.path.abspath(os.path.join("megatestresults", "py%s-ucs%d-sq%s" % (pyver, ucs, sqlitever)))
                run("mkdir -p %s/src %s/tools %s" % (workdir, workdir, logdir))
                run("cp *.py checksums "+workdir)
                run("cp tools/*.py "+workdir+"/tools/")
                run("cp src/*.c src/*.h "+workdir+"/src/")

                queue.put({'workdir': workdir, 'pyver': pyver, 'ucs': ucs, 'sqlitever': sqlitever, 'logdir': logdir})

    threads=[]
    for i in range(concurrency):
        queue.put(None) # exit sentinel
        t=threading.Thread(target=threadrun, args=(queue,))
        t.start()
        threads.append(t)

    print "All builds started, now waiting for them to finish (%d concurrency)" % (concurrency,)
    for t in threads:
        t.join()
    print "\nFinished"

def getpyurl(pyver):
    dirver=pyver
    if 'a' in dirver:
        dirver=dirver.split('a')[0]
    elif 'b' in dirver:
        dirver=dirver.split('b')[0]
    elif 'rc' in dirver:
        dirver=dirver.split('rc')[0]
    if pyver>'2.3.0':
        # Upper or lower case 'p' in download filename is somewhat random
        p='P'
        ext="bz2"
        # Python stopped making new releases as bz2 and instead it is
        # xz in the middle of a release stream
        switchvers=(
            "3.2.6",
            "2.7.7",
            "2.6.9",
        )
        v2i=lambda x: [int(i) for i in x.split(".")]
        if pyver>='3.3':
            ext="xz"
        for v in switchvers:
            if v2i(dirver)[:2]==v2i(v)[:2] and v2i(dirver)>=v2i(v):
                ext="xz"
                break
        return "https://www.python.org/ftp/python/%s/%sython-%s.tar.%s" % (dirver,p,pyver,ext)
    if pyver=='2.3.0':
        pyver='2.3'
        dirver='2.3'
    return "https://www.python.org/ftp/python/%s/Python-%s.tgz" % (dirver,pyver)

def buildpython(workdir, pyver, ucs, logfilename):
    if pyver=="system": return "/usr/bin/python", ""
    url=getpyurl(pyver)
    if url.endswith(".bz2"):
        tarx="j"
    elif url.endswith(".xz"):
        tarx="J"
    else:
        tarx="z"
    if pyver=="2.3.0": pyver="2.3"
    run("set -e ; cd %s ; mkdir pyinst ; ( echo \"Getting %s\"; wget -q %s -O - | tar xf%s -  ) > %s 2>&1" % (workdir, url, url, tarx, logfilename))
    # See https://bugs.launchpad.net/ubuntu/+source/gcc-defaults/+bug/286334
    if pyver.startswith("2.3"):
        # https://bugs.launchpad.net/bugs/286334
        opt='BASECFLAGS=-U_FORTIFY_SOURCE'
    else:
        opt=''
    if pyver.startswith("3.0"):
        full="full" # 3.1 rc 1 doesn't need 'fullinstall'
    else:
        full=""
    if sys.platform.startswith("linux"):
        # zlib on natty issue: http://lipyrary.blogspot.com/2011/05/how-to-compile-python-on-ubuntu-1104.html
        # LDFLAGS works for Python 2.5 onwards.  Edit setup on 2.3 and 2.4
        if pyver.startswith("2.3") or pyver.startswith("2.4"):
            patch_natty_build(os.path.join(workdir, "Python-"+pyver, "setup.py"))
        ldflags="LDFLAGS=\"-L/usr/lib/$(dpkg-architecture -qDEB_HOST_MULTIARCH)\"; export LDFLAGS;"
    else:
        ldflags=""
    run("set -e ; %s cd %s ; cd ?ython-%s ; ./configure %s --enable-unicode=ucs%d --prefix=%s/pyinst  >> %s 2>&1; make >>%s 2>&1; make  %sinstall >>%s 2>&1 ; make clean >/dev/null" % (ldflags, workdir, pyver, opt, ucs, workdir, logfilename, logfilename, full, logfilename))
    suf=""
    if pyver>="3.1":
        suf="3"
    pybin=os.path.join(workdir, "pyinst", "bin", "python"+suf)
    return pybin, os.path.join(workdir, "pyinst", "lib")

def patch_natty_build(setup):
    assert os.path.isfile(setup)
    out=[]
    for line in open(setup, "rtU"):
        if line.strip().startswith("lib_dirs = self.compiler.library_dirs + ["):
            t=" '/usr/lib/"+os.popen("dpkg-architecture -qDEB_HOST_MULTIARCH", "r").read().strip()+"', "
            i=line.index("[")
            line=line[:i+1]+t+line[i+1:]
        out.append(line)
    open(setup, "wt").write("".join(out))

# Default versions we support
PYVERS=(
    '3.7.0',
    '3.6.6',
    '3.5.6',
    '3.4.9',
    '3.3.7',
    '3.2.6',
    '3.1.5',
    '2.7.15',
    '2.6.9',
    '2.5.6',
    '2.4.6',
    '2.3.7',
    'system',
)

SQLITEVERS=(
    '3.25.0',
    '3.25.1',
    '3.25.2',
)

if __name__=='__main__':
    nprocs=0
    try:
        # try and work out how many processors there are - this works on linux
        for line in open("/proc/cpuinfo", "rt"):
            line=line.split()
            if line and line[0]=="processor":
                nprocs+=1
    except:
        pass
    # well there should be at least one!
    if nprocs==0:
        nprocs=1

    concurrency=nprocs*2
    if concurrency>8:
        concurrency=8

    parser=optparse.OptionParser()
    parser.add_option("--pyvers", dest="pyvers", help="Which Python versions to test against [%default]", default=",".join(PYVERS))
    parser.add_option("--sqlitevers", dest="sqlitevers", help="Which SQLite versions to test against [%default]", default=",".join(SQLITEVERS))
    parser.add_option("--fossil", dest="fossil", help="Also test current SQLite FOSSIL version [%default]", default=False, action="store_true")
    parser.add_option("--ucs", dest="ucs", help="Unicode character widths to test in bytes [%default]", default="2,4")
    parser.add_option("--tasks", dest="concurrency", help="Number of simultaneous builds/tests to run [%default]", default=concurrency)

    options,args=parser.parse_args()

    if args:
        parser.error("Unexpected options "+str(options))

    pyvers=options.pyvers.split(",")
    sqlitevers=options.sqlitevers.split(",")
    if options.fossil:
        sqlitevers.append("fossil")
    ucstest=[int(x) for x in options.ucs.split(",")]
    concurrency=int(options.concurrency)
    sqlitevers=[x for x in sqlitevers if x]
    main(pyvers, ucstest, sqlitevers, concurrency)




from __future__ import absolute_import
from allennlp.modules.matrix_attention.matrix_attention import MatrixAttention
from allennlp.modules.matrix_attention.bilinear_matrix_attention import BilinearMatrixAttention
from allennlp.modules.matrix_attention.cosine_matrix_attention import CosineMatrixAttention
from allennlp.modules.matrix_attention.dot_product_matrix_attention import DotProductMatrixAttention
from allennlp.modules.matrix_attention.legacy_matrix_attention import LegacyMatrixAttention
from allennlp.modules.matrix_attention.linear_matrix_attention import LinearMatrixAttention



# pylint: disable=invalid-name,no-self-use,protected-access


from __future__ import division
from __future__ import absolute_import
import math

from numpy.testing import assert_almost_equal
import torch

from allennlp.common.testing import AllenNlpTestCase
from allennlp.nn.decoding.decoder_trainers import MaximumMarginalLikelihood
from ..simple_transition_system import SimpleDecoderState, SimpleDecoderStep


class TestMaximumMarginalLikelihood(AllenNlpTestCase):
    def setUp(self):
        super(TestMaximumMarginalLikelihood, self).setUp()
        self.initial_state = SimpleDecoderState([0, 1],
                                                [[], []],
                                                [torch.Tensor([0.0]), torch.Tensor([0.0])],
                                                [0, 1])
        self.decoder_step = SimpleDecoderStep()
        self.targets = torch.Tensor([[[2, 3, 4], [1, 3, 4], [1, 2, 4]],
                                     [[3, 4, 0], [2, 3, 4], [0, 0, 0]]])
        self.target_mask = torch.Tensor([[[1, 1, 1], [1, 1, 1], [1, 1, 1]],
                                         [[1, 1, 0], [1, 1, 1], [0, 0, 0]]])

        self.supervision = (self.targets, self.target_mask)
        # High beam size ensures exhaustive search.
        self.trainer = MaximumMarginalLikelihood()

    def test_decode(self):
        decoded_info = self.trainer.decode(self.initial_state, self.decoder_step, self.supervision)

        # Our loss is the negative log sum of the scores from each target sequence.  The score for
        # each sequence in our simple transition system is just `-sequence_length`.
        instance0_loss = math.log(math.exp(-3) * 3)  # all three sequences have length 3
        instance1_loss = math.log(math.exp(-2) + math.exp(-3))  # one has length 2, one has length 3
        expected_loss = -(instance0_loss + instance1_loss) / 2
        assert_almost_equal(decoded_info[u'loss'].data.numpy(), expected_loss)



u"""
``KnowledgeGraphField`` is a ``Field`` which stores a knowledge graph representation.
"""


from __future__ import division
from __future__ import absolute_import
#typing
from collections import defaultdict

import editdistance
#overrides
import torch

from allennlp.common import util
from allennlp.common.checks import ConfigurationError
from allennlp.data.fields.field import Field
from allennlp.data.token_indexers.token_indexer import TokenIndexer
from allennlp.data.tokenizers.token import Token
from allennlp.data.tokenizers.tokenizer import Tokenizer
from allennlp.data.vocabulary import Vocabulary
from allennlp.nn import util as nn_util
from allennlp.semparse.contexts.knowledge_graph import KnowledgeGraph
try:
    from itertools import izip
except:
    izip = zip



class KnowledgeGraphField(Field):
    u"""
    A ``KnowledgeGraphField`` represents a ``KnowledgeGraph`` as a ``Field`` that can be used in a
    ``Model``.  For each entity in the graph, we output two things: a text representation of the
    entity, handled identically to a ``TextField``, and a list of linking features for each token
    in some input utterance.

    The output of this field is a dictionary::

        {
          "text": Dict[str, torch.Tensor],  # each tensor has shape (batch_size, num_entities, num_entity_tokens)
          "linking": torch.Tensor  # shape (batch_size, num_entities, num_utterance_tokens, num_features)
        }

    The ``text`` component of this dictionary is suitable to be passed into a
    ``TextFieldEmbedder`` (which handles the additional ``num_entities`` dimension without any
    issues).  The ``linking`` component of the dictionary can be used however you want to decide
    which tokens in the utterance correspond to which entities in the knowledge graph.

    In order to create the ``text`` component, we use the same dictionary of ``TokenIndexers``
    that's used in a ``TextField`` (as we're just representing the text corresponding to each
    entity).  For the ``linking`` component, we use a set of hard-coded feature extractors that
    operate between the text corresponding to each entity and each token in the utterance.

    Parameters
    ----------
    knowledge_graph : ``KnowledgeGraph``
        The knowledge graph that this field stores.
    utterance_tokens : ``List[Token]``
        The tokens in some utterance that is paired with the ``KnowledgeGraph``.  We compute a set
        of features for linking tokens in the utterance to entities in the graph.
    tokenizer : ``Tokenizer``
        We'll use this ``Tokenizer`` to tokenize the text representation of each entity.
    token_indexers : ``Dict[str, TokenIndexer]``
        Token indexers that convert entities into arrays, similar to how text tokens are treated in
        a ``TextField``.  These might operate on the name of the entity itself, its type, its
        neighbors in the graph, etc.
    feature_extractors : ``List[str]``, optional
        Names of feature extractors to use for computing linking features.  These must be
        attributes of this object, without the first underscore.  The feature extraction functions
        are listed as the last methods in this class.  For example, to use
        :func:`_exact_token_match`, you would pass the string ``exact_token_match``.  We will add
        an underscore and look for a function matching that name.  If this list is omitted, we will
        use all available feature functions.
    entity_tokens : ``List[List[Token]]``, optional
        If you have pre-computed the tokenization of the table text, you can pass it in here.  The
        must be a list of the tokens in the entity text, for each entity in the knowledge graph, in
        the same order in which the knowledge graph returns entities.
    linking_features : ``List[List[List[float]]]``, optional
        If you have pre-computed the linking features between the utterance and the table text, you
        can pass it in here.
    include_in_vocab : ``bool``, optional (default=True)
        If this is ``False``, we will skip the ``count_vocab_items`` logic, leaving out all table
        entity text from the vocabulary computation.  You might want to do this if you have a lot
        of rare entities in your tables, and you see the same table in multiple training instances,
        so your vocabulary counts get skewed and include too many rare entities.
    max_table_tokens : ``int``, optional
        If given, we will only keep this number of total table tokens.  This bounds the memory
        usage of the table representations, truncating cells with really long text.  We specify a
        total number of tokens, not a max cell text length, because the number of table entities
        varies.
    """
    def __init__(self,
                 knowledge_graph                ,
                 utterance_tokens             ,
                 token_indexers                         ,
                 tokenizer            = None,
                 feature_extractors            = None,
                 entity_tokens                    = None,
                 linking_features                          = None,
                 include_in_vocab       = True,
                 max_table_tokens      = None)        :
        self.knowledge_graph = knowledge_graph
        if not entity_tokens:
            entity_texts = [knowledge_graph.entity_text[entity].lower()
                            for entity in knowledge_graph.entities]
            # TODO(mattg): Because we do tagging on each of these entities in addition to just
            # tokenizations, this is quite slow, and about half of our data processing time just
            # goes to this (~15 minutes when there are 7k instances).  The reason we do tagging is
            # so that we can add lemma features.  If we can remove the need for lemma / other
            # hand-written features, like with a CNN, we can cut down our data processing time by a
            # factor of 2.
            self.entity_texts = tokenizer.batch_tokenize(entity_texts)
        else:
            self.entity_texts = entity_tokens
        self.utterance_tokens = utterance_tokens
        self._token_indexers = token_indexers
        self._include_in_vocab = include_in_vocab
        self._indexed_entity_texts = None
        self._max_table_tokens = max_table_tokens

        feature_extractors = feature_extractors if feature_extractors is not None else [
                u'number_token_match',
                u'exact_token_match',
                u'contains_exact_token_match',
                u'lemma_match',
                u'contains_lemma_match',
                u'edit_distance',
                u'related_column',
                u'related_column_lemma',
                u'span_overlap_fraction',
                u'span_lemma_overlap_fraction',
                ]
        self._feature_extractors = []
        for feature_extractor_name in feature_extractors:
            extractor = getattr(self, u'_' + feature_extractor_name, None)
            if not extractor:
                raise ConfigurationError("Invalid feature extractor name: {feature_extractor_name}")
            self._feature_extractors.append(extractor)

        if not linking_features:
            # For quicker lookups in our feature functions, we'll additionally store some
            # dictionaries that map entity strings to useful information about the entity.
            self._entity_text_map = {}
            for entity, entity_text in izip(knowledge_graph.entities, self.entity_texts):
                self._entity_text_map[entity] = entity_text

            self._entity_text_exact_text = {}
            for entity, entity_text in izip(knowledge_graph.entities, self.entity_texts):
                self._entity_text_exact_text[entity] = set(e.text for e in entity_text)

            self._entity_text_lemmas = {}
            for entity, entity_text in izip(knowledge_graph.entities, self.entity_texts):
                self._entity_text_lemmas[entity] = set(e.lemma_ for e in entity_text)
            self.linking_features = self._compute_linking_features()
        else:
            self.linking_features = linking_features

    #overrides
    def count_vocab_items(self, counter                           ):
        if self._include_in_vocab:
            for indexer in list(self._token_indexers.values()):
                for entity_text in self.entity_texts:
                    for token in entity_text:
                        indexer.count_vocab_items(token, counter)

    #overrides
    def index(self, vocab            ):
        self._indexed_entity_texts = {}
        for indexer_name, indexer in list(self._token_indexers.items()):
            indexer_arrays                  = defaultdict(list)

            for entity_text in self.entity_texts:
                for index_name, indexed in list(indexer.tokens_to_indices(entity_text, vocab, indexer_name).items()):
                    indexer_arrays[index_name].append(indexed)

            self._indexed_entity_texts.update(indexer_arrays)

    #overrides
    def get_padding_lengths(self)                  :
        num_entities = len(self.entity_texts)
        num_entity_tokens = max(len(entity_text) for entity_text in self.entity_texts)

        if self._max_table_tokens:
            # This truncates the number of entity tokens used, enabling larger tables (either in
            # the number of entities in the table, or the number of tokens per entity) to fit in
            # memory, particularly when using ELMo.
            if num_entities * num_entity_tokens > self._max_table_tokens:
                num_entity_tokens = int(self._max_table_tokens / num_entities)

        padding_lengths = {u'num_entities': num_entities,
                           u'num_utterance_tokens': len(self.utterance_tokens)}
        padding_lengths[u'num_entity_tokens'] = num_entity_tokens
        lengths = []
        assert self._indexed_entity_texts is not None, (u"This field is not indexed yet. Call "
                                                        u".index(vocab) before determining padding "
                                                        u"lengths.")
        for indexer_name, indexer in list(self._token_indexers.items()):
            indexer_lengths = {}

            # This is a list of dicts, one for each token in the field.
            entity_lengths = [indexer.get_padding_lengths(token)
                              for entity_text in self._indexed_entity_texts[indexer_name]
                              for token in entity_text]
            # Iterate over the keys in the first element of the list.  This is fine as for a given
            # indexer, all entities will return the same keys, so we can just use the first one.
            for key in list(entity_lengths[0].keys()):
                indexer_lengths[key] = max(x[key] if key in x else 0 for x in entity_lengths)
            lengths.append(indexer_lengths)

        # Get all the keys which have been used for padding.
        padding_keys = set([key for d in lengths for key in list(d.keys())])
        for padding_key in padding_keys:
            padding_lengths[padding_key] = max(x[padding_key] if padding_key in x else 0 for x in lengths)
        return padding_lengths

    #overrides
    def as_tensor(self,
                  padding_lengths                ,
                  cuda_device      = -1)                           :
        tensors = {}
        desired_num_entities = padding_lengths[u'num_entities']
        desired_num_entity_tokens = padding_lengths[u'num_entity_tokens']
        desired_num_utterance_tokens = padding_lengths[u'num_utterance_tokens']
        for indexer_name, indexer in list(self._token_indexers.items()):
            padded_entities = util.pad_sequence_to_length(self._indexed_entity_texts[indexer_name],
                                                          desired_num_entities,
                                                          default_value=lambda: [])
            padded_arrays = []
            for padded_entity in padded_entities:
                padded_array = indexer.pad_token_sequence({u'key': padded_entity},
                                                          {u'key': desired_num_entity_tokens},
                                                          padding_lengths)[u'key']
                padded_arrays.append(padded_array)
            tensor = torch.LongTensor(padded_arrays)
            tensors[indexer_name] = tensor if cuda_device == -1 else tensor.cuda(cuda_device)
        padded_linking_features = util.pad_sequence_to_length(self.linking_features,
                                                              desired_num_entities,
                                                              default_value=lambda: [])
        padded_linking_arrays = []
        default_feature_value = lambda: [0.0] * len(self._feature_extractors)
        for linking_features in padded_linking_features:
            padded_features = util.pad_sequence_to_length(linking_features,
                                                          desired_num_utterance_tokens,
                                                          default_value=default_feature_value)
            padded_linking_arrays.append(padded_features)
        linking_features_tensor = torch.FloatTensor(padded_linking_arrays)
        if cuda_device != -1:
            linking_features_tensor = linking_features_tensor.cuda(cuda_device)
        return {u'text': tensors, u'linking': linking_features_tensor}

    def _compute_linking_features(self)                           :
        linking_features = []
        for entity, entity_text in izip(self.knowledge_graph.entities, self.entity_texts):
            entity_features = []
            for token_index, token in enumerate(self.utterance_tokens):
                token_features = []
                for feature_extractor in self._feature_extractors:
                    token_features.append(feature_extractor(entity,
                                                            entity_text,
                                                            token,
                                                            token_index,
                                                            self.utterance_tokens))
                entity_features.append(token_features)
            linking_features.append(entity_features)
        return linking_features

    #overrides
    def empty_field(self)                         :
        return KnowledgeGraphField(KnowledgeGraph(set(), {}), [], self._token_indexers)

    #overrides
    def batch_tensors(self, tensor_list                               )                           :
        # pylint: disable=no-self-use
        batched_text = nn_util.batch_tensor_dicts(tensor[u'text'] for tensor in tensor_list)  # type: ignore
        batched_linking = torch.stack([tensor[u'linking'] for tensor in tensor_list])
        return {u'text': batched_text, u'linking': batched_linking}

    # Below here we have feature extractor functions.  To keep a consistent API for easy logic
    # above, some of these functions have unused arguments.
    # pylint: disable=unused-argument,no-self-use

    # These feature extractors are generally pretty specific to the logical form language and
    # problem setting in WikiTableQuestions.  This whole notion of feature extraction should
    # eventually be made more general (or just removed, if we can replace it with CNN features...).
    # For the feature functions used in the original parser written in PNP, see here:
    # https://github.com/allenai/pnp/blob/wikitables2/src/main/scala/org/allenai/wikitables/SemanticParserFeatureGenerator.scala

    # One notable difference between how the features work here and how they worked in PNP is that
    # we're using the table text when computing string matches, while PNP used the _entity name_.
    # It turns out that the entity name is derived from the table text, so this should be roughly
    # equivalent, except in the case of some numbers.  If there are cells with different text that
    # normalize to the same name, you could get `_2` or similar appended to the name, so the way we
    # do it here should just be better.  But it's a possible minor source of variation from the
    # original parser.

    # Another difference between these features and the PNP features is that the span overlap used
    # a weighting scheme to downweight matches on frequent words (like "the"), and the lemma
    # overlap feature value was calculated a little differently.  I'm guessing that doesn't make a
    # huge difference...

    def _number_token_match(self,
                            entity     ,
                            entity_text             ,
                            token       ,
                            token_index     ,
                            tokens             )         :
        # PNP had a "spanFeatures" function that said whether an entity was a-priori known to link
        # to a token or set of tokens in the question.  This was only used for numbers, and it's
        # not totally clear to me how this number feature overlapped with the token match features
        # in the original implementation (I think in most cases it was the same, except for things
        # like "four million", because the token match is derived from the entity name, which would
        # be 4000000, and wouldn't match "four million").
        #
        # Our implementation basically just adds a duplicate token match feature that's specific to
        # numbers.  It'll break in some rare cases (e.g., "Which four had four million ..."), but
        # those shouldn't be a big deal.
        if entity.startswith(u'fb:'):
            # This check works because numbers are the only entities that don't start with "fb:".
            return 0.0
        return self._contains_exact_token_match(entity, entity_text, token, token_index, tokens)

    def _exact_token_match(self,
                           entity     ,
                           entity_text             ,
                           token       ,
                           token_index     ,
                           tokens             )         :
        if len(entity_text) != 1:
            return 0.0
        return self._contains_exact_token_match(entity, entity_text, token, token_index, tokens)

    def _contains_exact_token_match(self,
                                    entity     ,
                                    entity_text             ,
                                    token       ,
                                    token_index     ,
                                    tokens             )         :
        if token.text in self._entity_text_exact_text[entity]:
            return 1.0
        return 0.0

    def _lemma_match(self,
                     entity     ,
                     entity_text             ,
                     token       ,
                     token_index     ,
                     tokens             )         :
        if len(entity_text) != 1:
            return 0.0
        return self._contains_lemma_match(entity, entity_text, token, token_index, tokens)

    def _contains_lemma_match(self,
                              entity     ,
                              entity_text             ,
                              token       ,
                              token_index     ,
                              tokens             )         :
        if token.text in self._entity_text_exact_text[entity]:
            return 1.0
        if token.lemma_ in self._entity_text_lemmas[entity]:
            return 1.0
        return 0.0

    def _edit_distance(self,
                       entity     ,
                       entity_text             ,
                       token       ,
                       token_index     ,
                       tokens             )         :
        edit_distance = float(editdistance.eval(u' '.join(e.text for e in entity_text), token.text))
        return 1.0 - edit_distance / len(token.text)

    def _related_column(self,
                        entity     ,
                        entity_text             ,
                        token       ,
                        token_index     ,
                        tokens             )         :
        if not entity.startswith(u'fb:row.row'):
            return 0.0
        for neighbor in self.knowledge_graph.neighbors[entity]:
            if token.text in self._entity_text_exact_text[neighbor]:
                return 1.0
        return 0.0

    def _related_column_lemma(self,
                              entity     ,
                              entity_text             ,
                              token       ,
                              token_index     ,
                              tokens             )         :
        if not entity.startswith(u'fb:row.row'):
            return 0.0
        for neighbor in self.knowledge_graph.neighbors[entity]:
            if token.text in self._entity_text_exact_text[neighbor]:
                return 1.0
            if token.lemma_ in self._entity_text_lemmas[neighbor]:
                return 1.0
        return 0.0

    def _span_overlap_fraction(self,
                               entity     ,
                               entity_text             ,
                               token       ,
                               token_index     ,
                               tokens             )         :
        entity_words = set(entity_token.text for entity_token in entity_text)
        if not entity_words:
            # Some tables have empty cells.
            return 0
        seen_entity_words = set()
        while token_index < len(tokens) and tokens[token_index].text in entity_words:
            seen_entity_words.add(tokens[token_index].text)
            token_index += 1
        return len(seen_entity_words) / len(entity_words)

    def _span_lemma_overlap_fraction(self,
                                     entity     ,
                                     entity_text             ,
                                     token       ,
                                     token_index     ,
                                     tokens             )         :
        entity_lemmas = set(entity_token.lemma_ for entity_token in entity_text)
        if not entity_lemmas:
            # Some tables have empty cells.
            return 0
        seen_entity_lemmas = set()
        while token_index < len(tokens) and tokens[token_index].lemma_ in entity_lemmas:
            seen_entity_lemmas.add(tokens[token_index].lemma_)
            token_index += 1
        return len(seen_entity_lemmas) / len(entity_lemmas)

    # pylint: enable=unused-argument,no-self-use



u"""
The ``evaluate`` subcommand can be used to
evaluate a trained model against a dataset
and report any metrics calculated by the model.

.. code-block:: bash

    $ allennlp evaluate --help
    usage: allennlp evaluate [-h] [--output-file OUTPUT_FILE]
                             [--weights-file WEIGHTS_FILE]
                             [--cuda-device CUDA_DEVICE] [-o OVERRIDES]
                             [--include-package INCLUDE_PACKAGE]
                             archive_file input_file

    Evaluate the specified model + dataset

    positional arguments:
    archive_file          path to an archived trained model
    input_file            path to the file containing the evaluation data

    optional arguments:
    -h, --help            show this help message and exit
    --output-file OUTPUT_FILE
                            path to output file to save metrics
    --weights-file WEIGHTS_FILE
                            a path that overrides which weights file to use
    --cuda-device CUDA_DEVICE
                            id of GPU to use (if any)
    -o OVERRIDES, --overrides OVERRIDES
                            a JSON structure used to override the experiment
                            configuration
    --include-package INCLUDE_PACKAGE
                            additional packages to include
"""


from __future__ import with_statement
from __future__ import absolute_import
#typing
import argparse
import logging
import json

import torch

from allennlp.commands.subcommand import Subcommand
from allennlp.common.checks import check_for_gpu
from allennlp.common.util import prepare_environment
from allennlp.common.tqdm import Tqdm
from allennlp.data import Instance
from allennlp.data.dataset_readers.dataset_reader import DatasetReader
from allennlp.data.iterators import DataIterator
from allennlp.models.archival import load_archive
from allennlp.models.model import Model
from io import open

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


class Evaluate(Subcommand):
    def add_subparser(self, name     , parser                            )                           :
        # pylint: disable=protected-access
        description = u'''Evaluate the specified model + dataset'''
        subparser = parser.add_parser(
                name, description=description, help=u'Evaluate the specified model + dataset')

        subparser.add_argument(u'archive_file', type=unicode, help=u'path to an archived trained model')

        subparser.add_argument(u'input_file', type=unicode, help=u'path to the file containing the evaluation data')

        subparser.add_argument(u'--output-file', type=unicode, help=u'path to output file')

        subparser.add_argument(u'--weights-file',
                               type=unicode,
                               help=u'a path that overrides which weights file to use')

        cuda_device = subparser.add_mutually_exclusive_group(required=False)
        cuda_device.add_argument(u'--cuda-device',
                                 type=int,
                                 default=-1,
                                 help=u'id of GPU to use (if any)')

        subparser.add_argument(u'-o', u'--overrides',
                               type=unicode,
                               default=u"",
                               help=u'a JSON structure used to override the experiment configuration')

        subparser.set_defaults(func=evaluate_from_args)

        return subparser


def evaluate(model       ,
             instances                    ,
             data_iterator              ,
             cuda_device     )                  :
    _warned_tqdm_ignores_underscores = False
    check_for_gpu(cuda_device)
    with torch.no_grad():
        model.eval()

        iterator = data_iterator(instances,
                                 num_epochs=1,
                                 shuffle=False,
                                 cuda_device=cuda_device)
        logger.info(u"Iterating over dataset")
        generator_tqdm = Tqdm.tqdm(iterator, total=data_iterator.get_num_batches(instances))
        for batch in generator_tqdm:
            model(**batch)
            metrics = model.get_metrics()
            if (not _warned_tqdm_ignores_underscores and
                        any(metric_name.startswith(u"_") for metric_name in metrics)):
                logger.warning(u"Metrics with names beginning with \"_\" will "
                               u"not be logged to the tqdm progress bar.")
                _warned_tqdm_ignores_underscores = True
            description = u', '.join([u"%s: %.2f" % (name, value) for name, value
                                     in list(metrics.items()) if not name.startswith(u"_")]) + u" ||"
            generator_tqdm.set_description(description, refresh=False)

        return model.get_metrics(reset=True)


def evaluate_from_args(args                    )                  :
    # Disable some of the more verbose logging statements
    logging.getLogger(u'allennlp.common.params').disabled = True
    logging.getLogger(u'allennlp.nn.initializers').disabled = True
    logging.getLogger(u'allennlp.modules.token_embedders.embedding').setLevel(logging.INFO)

    # Load from archive
    archive = load_archive(args.archive_file, args.cuda_device, args.overrides, args.weights_file)
    config = archive.config
    prepare_environment(config)
    model = archive.model
    model.eval()

    # Load the evaluation data

    # Try to use the validation dataset reader if there is one - otherwise fall back
    # to the default dataset_reader used for both training and validation.
    validation_dataset_reader_params = config.pop(u'validation_dataset_reader', None)
    if validation_dataset_reader_params is not None:
        dataset_reader = DatasetReader.from_params(validation_dataset_reader_params)
    else:
        dataset_reader = DatasetReader.from_params(config.pop(u'dataset_reader'))
    evaluation_data_path = args.input_file
    logger.info(u"Reading evaluation data from %s", evaluation_data_path)
    instances = dataset_reader.read(evaluation_data_path)

    iterator_params = config.pop(u"validation_iterator", None)
    if iterator_params is None:
        iterator_params = config.pop(u"iterator")
    iterator = DataIterator.from_params(iterator_params)
    iterator.index_with(model.vocab)

    metrics = evaluate(model, instances, iterator, args.cuda_device)

    logger.info(u"Finished evaluating.")
    logger.info(u"Metrics:")
    for key, metric in list(metrics.items()):
        logger.info(u"%s: %s", key, metric)

    output_file = args.output_file
    if output_file:
        with open(output_file, u"w") as file:
            json.dump(metrics, file, indent=4)
    return metrics





from __future__ import with_statement
from __future__ import absolute_import
#typing
import logging

#overrides

from allennlp.common.checks import ConfigurationError
from allennlp.common.file_utils import cached_path
from allennlp.common.util import START_SYMBOL, END_SYMBOL
from allennlp.data.dataset_readers.dataset_reader import DatasetReader
from allennlp.data.fields import TextField
from allennlp.data.instance import Instance
from allennlp.data.tokenizers import Token, Tokenizer, WordTokenizer
from allennlp.data.token_indexers import TokenIndexer, SingleIdTokenIndexer
from io import open

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


class Seq2SeqDatasetReader(DatasetReader):
    u"""
    Read a tsv file containing paired sequences, and create a dataset suitable for a
    ``SimpleSeq2Seq`` model, or any model with a matching API.

    Expected format for each input line: <source_sequence_string>\t<target_sequence_string>

    The output of ``read`` is a list of ``Instance`` s with the fields:
        source_tokens: ``TextField`` and
        target_tokens: ``TextField``

    `START_SYMBOL` and `END_SYMBOL` tokens are added to the source and target sequences.

    Parameters
    ----------
    source_tokenizer : ``Tokenizer``, optional
        Tokenizer to use to split the input sequences into words or other kinds of tokens. Defaults
        to ``WordTokenizer()``.
    target_tokenizer : ``Tokenizer``, optional
        Tokenizer to use to split the output sequences (during training) into words or other kinds
        of tokens. Defaults to ``source_tokenizer``.
    source_token_indexers : ``Dict[str, TokenIndexer]``, optional
        Indexers used to define input (source side) token representations. Defaults to
        ``{"tokens": SingleIdTokenIndexer()}``.
    target_token_indexers : ``Dict[str, TokenIndexer]``, optional
        Indexers used to define output (target side) token representations. Defaults to
        ``source_token_indexers``.
    source_add_start_token : bool, (optional, default=True)
        Whether or not to add `START_SYMBOL` to the beginning of the source sequence.
    """
    def __init__(self,
                 source_tokenizer            = None,
                 target_tokenizer            = None,
                 source_token_indexers                          = None,
                 target_token_indexers                          = None,
                 source_add_start_token       = True,
                 lazy       = False)        :
        super(Seq2SeqDatasetReader, self).__init__(lazy)
        self._source_tokenizer = source_tokenizer or WordTokenizer()
        self._target_tokenizer = target_tokenizer or self._source_tokenizer
        self._source_token_indexers = source_token_indexers or {u"tokens": SingleIdTokenIndexer()}
        self._target_token_indexers = target_token_indexers or self._source_token_indexers
        self._source_add_start_token = source_add_start_token

    #overrides
    def _read(self, file_path):
        with open(cached_path(file_path), u"r") as data_file:
            logger.info(u"Reading instances from lines in file at: %s", file_path)
            for line_num, line in enumerate(data_file):
                line = line.strip(u"\n")

                if not line:
                    continue

                line_parts = line.split(u'\t')
                if len(line_parts) != 2:
                    raise ConfigurationError(u"Invalid line format: %s (line number %d)" % (line, line_num + 1))
                source_sequence, target_sequence = line_parts
                yield self.text_to_instance(source_sequence, target_sequence)

    #overrides
    def text_to_instance(self, source_string     , target_string      = None)            :  # type: ignore
        # pylint: disable=arguments-differ
        tokenized_source = self._source_tokenizer.tokenize(source_string)
        if self._source_add_start_token:
            tokenized_source.insert(0, Token(START_SYMBOL))
        tokenized_source.append(Token(END_SYMBOL))
        source_field = TextField(tokenized_source, self._source_token_indexers)
        if target_string is not None:
            tokenized_target = self._target_tokenizer.tokenize(target_string)
            tokenized_target.insert(0, Token(START_SYMBOL))
            tokenized_target.append(Token(END_SYMBOL))
            target_field = TextField(tokenized_target, self._target_token_indexers)
            return Instance({u"source_tokens": source_field, u"target_tokens": target_field})
        else:
            return Instance({u'source_tokens': source_field})

Seq2SeqDatasetReader = DatasetReader.register(u"seq2seq")(Seq2SeqDatasetReader)



u"""
A logger that maintains logs of both stdout and stderr when models are run.
"""


from __future__ import absolute_import
#typing
import os
from io import open

def replace_cr_with_newline(message     ):
    u"""
    TQDM and requests use carriage returns to get the training line to update for each batch
    without adding more lines to the terminal output.  Displaying those in a file won't work
    correctly, so we'll just make sure that each batch shows up on its one line.
    :param message: the message to permute
    :return: the message with carriage returns replaced with newlines
    """
    if u'\r' in message:
        message = message.replace(u'\r', u'')
        if not message or message[-1] != u'\n':
            message += u'\n'
    return message

class TeeLogger(object):
    u"""
    This class is an attempt to maintain logs of both stdout and stderr for when models are run.
    To use this class, at the beginning of your script insert these lines::

        sys.stdout = TeeLogger("stdout.log", sys.stdout)
        sys.stderr = TeeLogger("stdout.log", sys.stderr)
    """
    def __init__(self, filename     , terminal        , file_friendly_terminal_output      )        :
        self.terminal = terminal
        self.file_friendly_terminal_output = file_friendly_terminal_output
        parent_directory = os.path.dirname(filename)
        os.makedirs(parent_directory, exist_ok=True)
        self.log = open(filename, u'a')

    def write(self, message):
        cleaned = replace_cr_with_newline(message)

        if self.file_friendly_terminal_output:
            self.terminal.write(cleaned)
        else:
            self.terminal.write(message)

        self.log.write(cleaned)

    def flush(self):
        self.terminal.flush()
        self.log.flush()





from __future__ import with_statement
from __future__ import absolute_import
#typing
import logging
import csv

#overrides

from allennlp.common.file_utils import cached_path
from allennlp.data.dataset_readers.dataset_reader import DatasetReader
from allennlp.data.fields import LabelField, TextField, Field
from allennlp.data.instance import Instance
from allennlp.data.tokenizers import Tokenizer, WordTokenizer
from allennlp.data.tokenizers.word_splitter import JustSpacesWordSplitter
from allennlp.data.token_indexers import TokenIndexer, SingleIdTokenIndexer
from io import open

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


class QuoraParaphraseDatasetReader(DatasetReader):
    u"""
    Reads a file from the Quora Paraphrase dataset. The train/validation/test split of the data
    comes from the paper `Bilateral Multi-Perspective Matching for Natural Language Sentences
    <https://arxiv.org/abs/1702.03814>`_ by Zhiguo Wang et al., 2017. Each file of the data
    is a tsv file without header. The columns are is_duplicate, question1, question2, and id.
    All questions are pre-tokenized and tokens are space separated. We convert these keys into
    fields named "label", "premise" and "hypothesis", so that it is compatible to some existing
    natural language inference algorithms.

    Parameters
    ----------
    lazy : ``bool`` (optional, default=False)
        Passed to ``DatasetReader``.  If this is ``True``, training will start sooner, but will
        take longer per batch.  This also allows training with datasets that are too large to fit
        in memory.
    tokenizer : ``Tokenizer``, optional
        Tokenizer to use to split the premise and hypothesis into words or other kinds of tokens.
        Defaults to ``WordTokenizer(JustSpacesWordSplitter())``.
    token_indexers : ``Dict[str, TokenIndexer]``, optional
        Indexers used to define input token representations. Defaults to ``{"tokens":
        SingleIdTokenIndexer()}``.
    """
    def __init__(self,
                 lazy       = False,
                 tokenizer            = None,
                 token_indexers                          = None)        :
        super(QuoraParaphraseDatasetReader, self).__init__(lazy)
        self._tokenizer = tokenizer or WordTokenizer(JustSpacesWordSplitter())
        self._token_indexers = token_indexers or {u"tokens": SingleIdTokenIndexer()}

    #overrides
    def _read(self, file_path):
        logger.info(u"Reading instances from lines in file at: %s", file_path)
        with open(cached_path(file_path), u"r") as data_file:
            tsv_in = csv.reader(data_file, delimiter=u'\t')
            for row in tsv_in:
                if len(row) == 4:
                    yield self.text_to_instance(premise=row[1], hypothesis=row[2], label=row[0])

    #overrides
    def text_to_instance(self,  # type: ignore
                         premise     ,
                         hypothesis     ,
                         label      = None)            :
        # pylint: disable=arguments-differ
        fields                   = {}
        tokenized_premise = self._tokenizer.tokenize(premise)
        tokenized_hypothesis = self._tokenizer.tokenize(hypothesis)
        fields[u"premise"] = TextField(tokenized_premise, self._token_indexers)
        fields[u"hypothesis"] = TextField(tokenized_hypothesis, self._token_indexers)
        if label is not None:
            fields[u'label'] = LabelField(label)

        return Instance(fields)

QuoraParaphraseDatasetReader = DatasetReader.register(u"quora_paraphrase")(QuoraParaphraseDatasetReader)



u"""
This module defines classes Object and Box (the two entities in the NLVR domain) and an NlvrWorld,
which mainly contains an execution method and related helper methods.
"""

from __future__ import absolute_import
from collections import defaultdict
import operator
#typing
import logging

from nltk.sem.logic import Type
#overrides

from allennlp.common.util import JsonDict
from allennlp.semparse import util as semparse_util
from allennlp.semparse.worlds.world import ExecutionError
from allennlp.semparse.type_declarations import nlvr_type_declaration as types
from allennlp.semparse.worlds.world import World

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name

# typevar


class Object(object):
    u"""
    ``Objects`` are the geometric shapes in the NLVR domain. They have values for attributes shape,
    color, x_loc, y_loc and size. We take a dict read from the JSON file and store it here, and
    define a get method for getting the attribute values. We need this to be hashable because need
    to make sets of ``Objects`` during execution, which get passed around between functions.

    Parameters
    ----------
    attributes : ``JsonDict``
        The dict for each object from the json file.
    """
    def __init__(self, attributes          , box_id     )        :
        object_color = attributes[u"color"].lower()
        # The dataset has a hex code only for blue for some reason.
        if object_color.startswith(u"#"):
            self.color = u"blue"
        else:
            self.color = object_color
        object_shape = attributes[u"type"].lower()
        self.shape = object_shape
        self.x_loc = attributes[u"x_loc"]
        self.y_loc = attributes[u"y_loc"]
        self.size = attributes[u"size"]
        self._box_id = box_id

    def __str__(self):
        if self.size == 10:
            size = u"small"
        elif self.size == 20:
            size = u"medium"
        else:
            size = u"big"
        return "{size} {self.color} {self.shape} at ({self.x_loc}, {self.y_loc}) in {self._box_id}"

    def __hash__(self):
        return hash(unicode(self))

    def __eq__(self, other):
        return unicode(self) == unicode(other)


class Box(object):
    u"""
    This class represents each box containing objects in NLVR.

    Parameters
    ----------
    objects_list : ``List[JsonDict]``
        List of objects in the box, as given by the json file.
    box_id : ``int``
        An integer identifying the box index (0, 1 or 2).
    """
    def __init__(self,
                 objects_list                ,
                 box_id     )        :
        self._name = "box {box_id + 1}"
        self.objects = set([Object(object_dict, self._name) for object_dict in objects_list])

    def __str__(self):
        return self._name

    def __hash__(self):
        return hash(unicode(self))

    def __eq__(self, other):
        return unicode(self) == unicode(other)


class NlvrWorld(World):
    u"""
    Class defining the world representation of NLVR. Defines an execution logic for logical forms
    in NLVR.  We just take the structured_rep from the JSON file to initialize this.

    Parameters
    ----------
    world_representation : ``JsonDict``
        structured_rep from the JSON file.
    """
    # pylint: disable=too-many-public-methods

    # When we're converting from logical forms to action sequences, this set tells us which
    # functions in the logical form are curried functions, and how many arguments the function
    # actually takes.  This is necessary because NLTK curries all multi-argument functions to a
    # series of one-argument function applications.  See `world._get_transitions` for more info.
    curried_functions = {
            types.BOX_COLOR_FILTER_TYPE: 2,
            types.BOX_SHAPE_FILTER_TYPE: 2,
            types.BOX_COUNT_FILTER_TYPE: 2,
            types.ASSERT_COLOR_TYPE: 2,
            types.ASSERT_SHAPE_TYPE: 2,
            types.ASSERT_BOX_COUNT_TYPE: 2,
            types.ASSERT_OBJECT_COUNT_TYPE: 2,
            }

    # TODO(pradeep): Define more spatial relationship methods: left_of, right_of..
    # They should be defined for objects within the same box.
    def __init__(self, world_representation                      )        :
        super(NlvrWorld, self).__init__(global_type_signatures=types.COMMON_TYPE_SIGNATURE,
                                        global_name_mapping=types.COMMON_NAME_MAPPING,
                                        num_nested_lambdas=0)
        self._boxes = set([Box(object_list, box_id) for box_id, object_list in
                           enumerate(world_representation)])
        self._objects =  set()
        for box in self._boxes:
            self._objects.update(box.objects)

        self._number_operators = {u"equals": operator.eq,
                                  u"not_equals": operator.ne,
                                  u"greater": operator.gt,
                                  u"lesser": operator.lt,
                                  u"greater_equals": operator.ge,
                                  u"lesser_equals": operator.le}

        self._set_unary_operators = {u"same": self._same,
                                     u"different": self._different}

        self._set_binary_operators = {u"all_equals": self._all_equals,
                                      u"any_equals": self._any_equals,
                                      u"none_equals": self._none_equals}

        self._count_functions = {u"count": self._count,  # type: ignore
                                 u"color_count": self._color_count,
                                 u"shape_count": self._shape_count}

        self._attribute_functions = {u"shape": self._shape,
                                     u"color": self._color}

        # Mapping from terminal strings to productions that produce them.
        # Eg.: "yellow" -> "<o,o> -> yellow", "<b,<<b,e>,<e,b>>> -> filter_greater" etc.
        self.terminal_productions =  {}
        for constant in types.COMMON_NAME_MAPPING:
            alias = types.COMMON_NAME_MAPPING[constant]
            if alias in types.COMMON_TYPE_SIGNATURE:
                constant_type = types.COMMON_TYPE_SIGNATURE[alias]
                self.terminal_productions[constant] = u"%s -> %s" % (constant_type, constant)

    #overrides
    def get_basic_types(self)             :
        return types.BASIC_TYPES

    #overrides
    def get_valid_starting_types(self)             :
        return set([types.TRUTH_TYPE])

    def _get_curried_functions(self)                   :
        return NlvrWorld.curried_functions

    #overrides
    def _map_name(self, name     , keep_mapping       = False)       :
        return types.COMMON_NAME_MAPPING[name] if name in types.COMMON_NAME_MAPPING else name

    def get_agenda_for_sentence(self,
                                sentence     ,
                                add_paths_to_agenda       = False)             :
        u"""
        Given a ``sentence``, returns a list of actions the sentence triggers as an ``agenda``. The
        ``agenda`` can be used while by a parser to guide the decoder.  sequences as possible. This
        is a simplistic mapping at this point, and can be expanded.

        Parameters
        ----------
        sentence : ``str``
            The sentence for which an agenda will be produced.
        add_paths_to_agenda : ``bool`` , optional
            If set, the agenda will also include nonterminal productions that lead to the terminals
            from the root node (default = False).
        """
        agenda = []
        sentence = sentence.lower()
        if sentence.startswith(u"there is a box") or sentence.startswith(u"there is a tower "):
            agenda.append(self.terminal_productions[u"box_exists"])
        elif sentence.startswith(u"there is a "):
            agenda.append(self.terminal_productions[u"object_exists"])

        if u"<b,t> -> box_exists" not in agenda:
            # These are object filters and do not apply if we have a box_exists at the top.
            if u"touch" in sentence:
                if u"top" in sentence:
                    agenda.append(self.terminal_productions[u"touch_top"])
                elif u"bottom" in sentence or u"base" in sentence:
                    agenda.append(self.terminal_productions[u"touch_bottom"])
                elif u"corner" in sentence:
                    agenda.append(self.terminal_productions[u"touch_corner"])
                elif u"right" in sentence:
                    agenda.append(self.terminal_productions[u"touch_right"])
                elif u"left" in sentence:
                    agenda.append(self.terminal_productions[u"touch_left"])
                elif u"wall" in sentence or u"edge" in sentence:
                    agenda.append(self.terminal_productions[u"touch_wall"])
                else:
                    agenda.append(self.terminal_productions[u"touch_object"])
            else:
                # The words "top" and "bottom" may be referring to top and bottom blocks in a tower.
                if u"top" in sentence:
                    agenda.append(self.terminal_productions[u"top"])
                elif u"bottom" in sentence or u"base" in sentence:
                    agenda.append(self.terminal_productions[u"bottom"])

            if u" not " in sentence:
                agenda.append(self.terminal_productions[u"negate_filter"])

        if u" contains " in sentence or u" has " in sentence:
            agenda.append(self.terminal_productions[u"all_boxes"])
        # This takes care of shapes, colors, top, bottom, big, small etc.
        for constant, production in list(self.terminal_productions.items()):
            # TODO(pradeep): Deal with constant names with underscores.
            if u"top" in constant or u"bottom" in constant:
                # We already dealt with top, bottom, touch_top and touch_bottom above.
                continue
            if constant in sentence:
                if u"<o,o> ->" in production and u"<b,t> -> box_exists" in agenda:
                    if constant in [u"square", u"circle", u"triangle"]:
                        agenda.append(self.terminal_productions["shape_{constant}"])
                    elif constant in [u"yellow", u"blue", u"black"]:
                        agenda.append(self.terminal_productions["color_{constant}"])
                    else:
                        continue
                else:
                    agenda.append(production)
        # TODO (pradeep): Rules for "member_*" productions ("tower" or "box" followed by a color,
        # shape or number...)
        number_productions = self._get_number_productions(sentence)
        for production in number_productions:
            agenda.append(production)
        if not agenda:
            # None of the rules above was triggered!
            if u"box" in sentence:
                agenda.append(self.terminal_productions[u"all_boxes"])
            else:
                agenda.append(self.terminal_productions[u"all_objects"])
        if add_paths_to_agenda:
            agenda = self._add_nonterminal_productions(agenda)
        return agenda

    @staticmethod
    def _get_number_productions(sentence     )             :
        u"""
        Gathers all the numbers in the sentence, and returns productions that lead to them.
        """
        # The mapping here is very simple and limited, which also shouldn't be a problem
        # because numbers seem to be represented fairly regularly.
        number_strings = {u"one": u"1", u"two": u"2", u"three": u"3", u"four": u"4", u"five": u"5", u"six":
                          u"6", u"seven": u"7", u"eight": u"8", u"nine": u"9", u"ten": u"10"}
        number_productions = []
        tokens = sentence.split()
        numbers = list(number_strings.values())
        for token in tokens:
            if token in numbers:
                number_productions.append("e -> {token}")
            elif token in number_strings:
                number_productions.append("e -> {number_strings[token]}")
        return number_productions

    def _add_nonterminal_productions(self, agenda           )             :
        u"""
        Given a partially populated agenda with (mostly) terminal productions, this method adds the
        nonterminal productions that lead from the root to the terminal productions.
        """
        nonterminal_productions = set(agenda)
        for action in agenda:
            paths = self.get_paths_to_root(action, max_num_paths=5)
            for path in paths:
                for path_action in path:
                    nonterminal_productions.add(path_action)
        new_agenda = list(nonterminal_productions)
        return new_agenda

    ## Complex operators
    @staticmethod
    def _same(input_set          )        :
        return len(input_set) == 1

    @staticmethod
    def _different(input_set          )        :
        return len(input_set) != 1

    @staticmethod
    def _all_equals(input_set          , target_value)        :
        if not input_set:
            return False
        return all([x == target_value for x in input_set])

    @staticmethod
    def _any_equals(input_set          , target_value)        :
        return any([x == target_value for x in input_set])

    ## Attribute functions
    @staticmethod
    def _color(objects             )            :
        u"""
        Returns the set of colors of a set of objects.
        """
        return set([obj.color for obj in objects])

    @staticmethod
    def _shape(objects             )            :
        u"""
        Returns the set of shapes of a set of objects.
        """
        return set([obj.shape for obj in objects])

    @staticmethod
    def _count(entities_set                              )       :
        return len(entities_set)

    @staticmethod
    def _none_equals(input_set          , target_value)        :
        return all([x != target_value for x in input_set])

    @classmethod
    def _shape_count(cls, input_set             )       :
        return len(cls._shape(input_set))

    @classmethod
    def _color_count(cls, input_set             )       :
        return len(cls._color(input_set))

    def execute(self, logical_form     )        :
        u"""
        Execute the logical form. The top level function is an assertion function (see below). We
        just parse the string into a list and pass the whole thing to ``_execute_assertion`` and let
        the method deal with it. This is because the dataset contains sentences (instead of
        questions), and they evaluate to either true or false.

        The language we defined here contains six types of functions, five of which return sets,
        and one returns booleans.

        1) Assertion Function : These occur only at the root node of the logical form trees. They
        take a set of entities, and compare their attributes to a given value, and return true or
        false. The entities they take can be boxes or objects. If the assertion function takes
        objects, it may compare their colors or shapes with the given value; If it takes boxes,
        the attributes it compares are only the counts. The comparison operator can be any of
        equals, not equals, greater than, etc. So, the function specifies what kind of entities it
        takes, the attribute being compared and the comparison operator. For example,
        "object_count_not_equals" takes a set of objects, compares their count to the given value
        and returns true iff they are not equal. They have names like "object_*" or "box_*"

        2) Object Attribute Functions: They take sets of objects and return sets of attributes.
        `color` and `shape` are the attribute functions.

        3) Box Membership Function : This takes a box as an argument and returns the objects in it.
        This is a special kind of attribute function for two reasons. Firstly, it returns a set of
        objects instead of attributes, and secondly it occurs only within the second argument of a
        box filtering function (see below). It provides a way to query boxes based on the
        attributes of objects contained within it. The function is called ``object_in_box``, and it
        gets executed within ``_execute_box_filter``.

        4) Box Filtering Functions : These are of the form `filter(set_of_boxes,
        attribute_function, target_attribute)` The idea is that we take a set of boxes, an
        attribute function that extracts the relevant attribute from a box, and a target attribute
        that we compare against. The logic is that we execute the attribute function on `each` of
        the given boxes and return only those whose attribute value, in comparison with the target
        attribute, satisfies the filtering criterion (i.e., equal to the target, less than, greater
        than etc.). The fitering function defines the comparison operator.  All the functions in
        this class with names ``filter_*`` belong to this category.

        5) Object Filtering Functions : These are of the form ``filter(set_of_objects)``. These are
        similar to box filtering functions, but they operate on objects instead. Also, note that
        they take just one argument instead of three. This is because while box filtering functions
        typically query complex attributes, object filtering functions query the properties of the
        objects alone.  These are simple and finite in number. Thus, we essentially let the
        filtering function define the attribute function, and the target attribute as well, along
        with the comparison operator.  That is, these are functions like `black` (which takes a set
        of objects, and returns those whose "color" (attribute function) "equals" (comparison
        operator) "black" (target attribute)), or "square" (which returns objects that are
        squares).

        6) Negate Object Filter : Takes an object filter and a set of objects and applies the
        negation of the object filter on the set.
        """
        if not logical_form.startswith(u"("):
            logical_form = u"(%s)" % logical_form
        logical_form = logical_form.replace(u",", u" ")
        expression_as_list = semparse_util.lisp_to_nested_expression(logical_form)
        # The whole expression has to be an assertion expression because it has to return a boolean.
        # TODO(pradeep): May want to make this more general and let the executor deal with questions.
        return self._execute_assertion(expression_as_list)

    # TODO(pradeep): The methods ``_execute_assertion``, ``_execute_box_filter`` and
    # ``execute_object_filter`` are very complex a this point. I should break these down into a
    # simpler set of functions, such that one there is a method for each terminal, and the
    # "execution logic" is minimal.

    def _execute_assertion(self, sub_expression      )        :
        u"""
        Assertion functions are boolean functions. They are of two types:
        1) Exists functions: They take one argument, a set and check whether it is not empty.
        Syntax: ``(exists_function function_returning_entities)``
        Example: ``(object_exists (black (top all_objects)))`` ("There is a black object at the top
        of a tower.")
        2) Other assert functions: They take two arguments, which evaluate to strings or integers,
        and compare them. The first element in the input list should be the assertion function name,
        the second a function returning entities, and the last element should be a constant. The
        assertion function should specify the entity type, the attribute being compared, and a
        comparison operator, in that order separated by underscores. The following are the expected
        values:
            Entity types: ``object``, ``box``
            Attributes being compared: ``color``, ``shape``, ``count``, ``color_count``,
            ``shape_count``
            Comparison operator:
                Applicable to sets: ``all_equals``, ``any_equals``, ``none_equals``, ``same``,
                ``different``
                Applicable to counts: ``equals``, ``not_equals``, ``lesser``, ``lesser_equals``,
                ``greater``, ``greater_equals``
        Syntax: ``(assertion_function function_returning_entities constant)``
        Example: ``(box_count_equals (member_shape_equals all_boxes shape_square) 2)``
        ("There are exactly two boxes with only squares in them")

        Note that the first kind is a special case of the second where the attribute type is
        ``count``, comparison operator is ``greater_equals`` and the constant is ``1``.
        """
        # TODO(pradeep): We may want to change the order of arguments here to make decoding easier.
        assert isinstance(sub_expression, list), u"Invalid assertion expression: %s" % sub_expression
        if len(sub_expression) == 1 and isinstance(sub_expression[0], list):
            return self._execute_assertion(sub_expression[0])
        is_assert_function = sub_expression[0].startswith(u"object_") or\
        sub_expression[0].startswith(u"box_")
        assert isinstance(sub_expression[0], unicode) and is_assert_function,\
               u"Invalid assertion function: %s" % (sub_expression[0])
        # Example: box_count_not_equals, entities being evaluated are boxes, the relevant attibute
        # is their count, and the function will return true if the attribute is not equal to the
        # target.
        function_name_parts = sub_expression[0].split(u'_')
        entity_type = function_name_parts[0]
        target_attribute = None
        if len(function_name_parts) == 2 and function_name_parts[1] == u"exists":
            attribute_type = u"count"
            comparison_op = u"greater_equals"
            target_attribute = 1
        else:
            target_attribute = self._execute_constant(sub_expression[2])
            # If the length of ``function_name_parts`` is 3, getting the attribute and comparison
            # operator is easy. However, if it is greater than 3, we need to determine where the
            # attribute function stops and where the comparison operator begins.
            if len(function_name_parts) == 3:
                # These are cases like ``object_color_equals``, ``box_count_greater`` etc.
                attribute_type = function_name_parts[1]
                comparison_op = function_name_parts[2]
            elif function_name_parts[2] == u'count':
                # These are cases like ``object_color_count_equals``,
                # ``object_shape_count_greater_equals`` etc.
                attribute_type = u"_".join(function_name_parts[1:3])
                comparison_op = u"_".join(function_name_parts[3:])
            else:
                # These are cases like ``box_count_greater_equals``, ``object_shape_not_equals``
                # etc.
                attribute_type = function_name_parts[1]
                comparison_op = u"_".join(function_name_parts[2:])

        entity_expression = sub_expression[1]
        returned_count = None
        returned_attribute = None
        if entity_type == u"box":
            # You can only count boxes. The other attributes do not apply.
            returned_count = self._count(self._execute_box_filter(entity_expression))
        elif u"count" in attribute_type:
            # We're counting objects, colors or shapes.
            count_function = self._count_functions[attribute_type]
            returned_count = count_function(self._execute_object_filter(entity_expression))
        else:
            # We're getting colors or shapes from objects.
            attribute_function = self._attribute_functions[attribute_type]
            returned_attribute = attribute_function(self._execute_object_filter(entity_expression))

        if comparison_op in [u"all_equals", u"any_equals", u"none_equals"]:
            set_comparison = self._set_binary_operators[comparison_op]
            if returned_attribute is None:
                logger.error(u"Invalid assertion function: %s", sub_expression[0])
                raise ExecutionError(u"Invalid assertion function")
            return set_comparison(returned_attribute, target_attribute)
        else:
            number_comparison = self._number_operators[comparison_op]
            if returned_count is None:
                logger.error(u"Invalid assertion function: %s", sub_expression[0])
                raise ExecutionError(u"Invalid assertion function")
            return number_comparison(returned_count, target_attribute)

    def _execute_box_filter(self, sub_expression                  )            :
        u"""
        Box filtering functions either apply a filter on a set of boxes and return the filtered set,
        or return all the boxes.
        The elements should evaluate to one of the following:
        ``(box_filtering_function set_to_filter constant)`` or
        ``all_boxes``

        In the first kind of forms, the ``box_filtering_function`` also specifies the attribute
        being compared and the comparison operator. The attribute is of the objects contained in
        each box in the ``set_to_filter``.
        Example: ``(member_color_count_greater all_boxes 1)``
        filters all boxes by extracting the colors of the objects in each of them, and returns a
        subset of boxes from the original set where the number of colors of objects is greater than
        1.
        """
        # TODO(pradeep): We may want to change the order of arguments here to make decoding easier.
        if sub_expression[0].startswith(u'member_'):
            function_name_parts = sub_expression[0].split(u"_")
            if len(function_name_parts) == 3:
                attribute_type = function_name_parts[1]
                comparison_op = function_name_parts[2]
            elif function_name_parts[2] == u"count":
                attribute_type = u"_".join(function_name_parts[1:3])
                comparison_op = u"_".join(function_name_parts[3:])
            else:
                attribute_type = function_name_parts[1]
                comparison_op = u"_".join(function_name_parts[2:])
            set_to_filter = self._execute_box_filter(sub_expression[1])
            return_set = set()
            if comparison_op in [u"same", u"different"]:
                # We don't need a target attribute for these functions, and the "comparison" is done
                # on sets.
                comparison_function = self._set_unary_operators[comparison_op]
                for box in set_to_filter:
                    returned_attribute           = self._attribute_functions[attribute_type](box.objects)
                    if comparison_function(returned_attribute):
                        return_set.add(box)
            else:
                target_attribute = self._execute_constant(sub_expression[-1])
                is_set_operation = comparison_op in [u"all_equals", u"any_equals", u"none_equals"]
                # These are comparisons like equals, greater etc, and we need a target attribute
                # which we first evaluate here. Then, the returned attribute (if it is a singleton
                # set or an integer), is compared against the target attribute.
                for box in set_to_filter:
                    if is_set_operation:
                        returned_attribute = self._attribute_functions[attribute_type](box.objects)
                        box_wanted = self._set_binary_operators[comparison_op](returned_attribute,
                                                                               target_attribute)
                    else:
                        returned_count = self._count_functions[attribute_type](box.objects)
                        box_wanted = self._number_operators[comparison_op](returned_count,
                                                                           target_attribute)
                    if box_wanted:
                        return_set.add(box)
            return return_set
        elif sub_expression == u'all_boxes' or sub_expression[0] == u'all_boxes':
            return self._boxes
        else:
            logger.error(u"Invalid box filter expression: %s", sub_expression)
            raise ExecutionError(u"Unknown box filter expression")

    def _execute_object_filter(self, sub_expression                  )               :
        u"""
        Object filtering functions should either be a string referring to all objects, or list which
        executes to a filtering operation.
        The elements should evaluate to one of the following:
            (object_filtering_function object_set)
            ((negate_filter object_filtering_function) object_set)
            all_objects
        """
        if sub_expression[0][0] == u"negate_filter":
            initial_set = self._execute_object_filter(sub_expression[1])
            original_filter_name = sub_expression[0][1]
            # It is possible that the decoder has produced a sequence of nested negations. We deal
            # with that here.
            # TODO (pradeep): This is messy. Fix the type declaration so that we don't have to deal
            # with this.
            num_negations = 1
            while isinstance(original_filter_name, list) and\
                  original_filter_name[0] == u"negate_filter":
                # We have a sequence of "negate_filters"
                num_negations += 1
                original_filter_name = original_filter_name[1]
            if num_negations % 2 == 0:
                return initial_set
            try:
                original_filter = getattr(self, original_filter_name)
                return self.negate_filter(original_filter, initial_set)
            except AttributeError:
                logger.error(u"Function not found: %s", original_filter_name)
                raise ExecutionError(u"Function not found")
        elif sub_expression == u"all_objects" or sub_expression[0] == u"all_objects":
            return self._objects
        elif isinstance(sub_expression[0], unicode) and len(sub_expression) == 2:
            # These are functions like black, square, same_color etc.
            function = None
            try:
                function = getattr(self, sub_expression[0])
            except AttributeError:
                logger.error(u"Function not found: %s", sub_expression[0])
                raise ExecutionError(u"Function not found")
            arguments = sub_expression[1]
            if isinstance(arguments, list) and unicode(arguments[0]).startswith(u"member_") or\
                arguments == u'all_boxes' or arguments[0] == u'all_boxes':
                if sub_expression[0] != u"object_in_box":
                    logger.error(u"Invalid object filter expression: %s", sub_expression)
                    raise ExecutionError(u"Invalid object filter expression")
                return function(self._execute_box_filter(arguments))
            else:
                return function(self._execute_object_filter(arguments))
        else:
            logger.error(u"Invalid object filter expression: %s", sub_expression)
            raise ExecutionError(u"Invalid object filter expression")

    @staticmethod
    def _execute_constant(sub_expression     ):
        u"""
        Acceptable constants are numbers or strings starting with `shape_` or `color_`
        """
        if not isinstance(sub_expression, unicode):
            logger.error(u"Invalid constant: %s", sub_expression)
            raise ExecutionError(u"Invalid constant")
        if unicode.isdigit(sub_expression):
            return int(sub_expression)
        elif sub_expression.startswith(u'color_'):
            return sub_expression.replace(u'color_', u'')
        elif sub_expression.startswith(u'shape_'):
            return sub_expression.replace(u'shape_', u'')
        else:
            logger.error(u"Invalid constant: %s", sub_expression)
            raise ExecutionError(u"Invalid constant")

    @staticmethod
    def _filter_boxes(set_to_filter          ,
                      attribute_function                                ,
                      target_attribute               ,
                      comparison_op                                                )            :
        returned_set = set()
        for entity in set_to_filter:
            if comparison_op(attribute_function(entity), target_attribute):
                returned_set.add(entity)
        return returned_set

    @staticmethod
    def object_in_box(box          )               :
        return_set              = set()
        for box_ in box:
            return_set.update(box_.objects)
        return return_set

    ## Object filtering functions
    @classmethod
    def black(cls, objects             )               :
        return set([obj for obj in objects if obj.color == u"black"])

    @classmethod
    def blue(cls, objects             )               :
        return set([obj for obj in objects if obj.color == u"blue"])

    @classmethod
    def yellow(cls, objects             )               :
        return set([obj for obj in objects if obj.color == u"yellow"])

    @classmethod
    def circle(cls, objects             )               :
        return set([obj for obj in objects if obj.shape == u"circle"])

    @classmethod
    def square(cls, objects             )               :
        return set([obj for obj in objects if obj.shape == u"square"])

    @classmethod
    def triangle(cls, objects             )               :
        return set([obj for obj in objects if obj.shape == u"triangle"])

    @staticmethod
    def _get_objects_with_same_attribute(objects             ,
                                         attribute_function                         )               :
        u"""
        Returns the set of objects for which the attribute function returns an attribute value that
        is most frequent in the initial set, if the frequency is greater than 1. If not, all
        objects have different attribute values, and this method returns an empty set.
        """
        objects_of_attribute                         = defaultdict(set)
        for entity in objects:
            objects_of_attribute[attribute_function(entity)].add(entity)
        if not objects_of_attribute:
            return set()
        most_frequent_attribute = max(objects_of_attribute, key=lambda x: len(objects_of_attribute[x]))
        if len(objects_of_attribute[most_frequent_attribute]) <= 1:
            return set()
        return objects_of_attribute[most_frequent_attribute]

    @classmethod
    def same_color(cls, objects             )               :
        u"""
        Filters the set of objects, and returns those objects whose color is the most frequent
        color in the initial set of objects, if the highest frequency is greater than 1, or an
        empty set otherwise.

        This is an unusual name for what the method does, but just as ``blue`` filters objects to
        those that are blue, this filters objects to those that are of the same color.
        """
        return cls._get_objects_with_same_attribute(objects, lambda x: x.color)

    @classmethod
    def same_shape(cls, objects             )               :
        u"""
        Filters the set of objects, and returns those objects whose color is the most frequent
        color in the initial set of objects, if the highest frequency is greater than 1, or an
        empty set otherwise.

        This is an unusual name for what the method does, but just as ``triangle`` filters objects
        to those that are triangles, this filters objects to those that are of the same shape.
        """
        return cls._get_objects_with_same_attribute(objects, lambda x: x.shape)

    @classmethod
    def touch_bottom(cls, objects             )               :
        return set([obj for obj in objects if obj.y_loc + obj.size == 100])

    @classmethod
    def touch_left(cls, objects             )               :
        return set([obj for obj in objects if obj.x_loc == 0])

    @classmethod
    def touch_top(cls, objects             )               :
        return set([obj for obj in objects if obj.y_loc == 0])

    @classmethod
    def touch_right(cls, objects             )               :
        return set([obj for obj in objects if obj.x_loc + obj.size == 100])

    @classmethod
    def touch_wall(cls, objects             )               :
        return_set              = set()
        return return_set.union(cls.touch_top(objects), cls.touch_left(objects),
                                cls.touch_right(objects), cls.touch_bottom(objects))

    @classmethod
    def touch_corner(cls, objects             )               :
        return_set              = set()
        return return_set.union(cls.touch_top(objects).intersection(cls.touch_right(objects)),
                                cls.touch_top(objects).intersection(cls.touch_left(objects)),
                                cls.touch_bottom(objects).intersection(cls.touch_right(objects)),
                                cls.touch_bottom(objects).intersection(cls.touch_left(objects)))

    def touch_object(self, objects             )               :
        u"""
        Returns all objects that touch the given set of objects.
        """
        objects_per_box = self._separate_objects_by_boxes(objects)
        return_set = set()
        for box, box_objects in list(objects_per_box.items()):
            candidate_objects = box.objects
            for object_ in box_objects:
                for candidate_object in candidate_objects:
                    if self._objects_touch_each_other(object_, candidate_object):
                        return_set.add(candidate_object)
        return return_set

    @classmethod
    def _objects_touch_each_other(cls, object1        , object2        )        :
        u"""
        Returns true iff the objects touch each other.
        """
        in_vertical_range = object1.y_loc <= object2.y_loc + object2.size and\
                            object1.y_loc + object1.size >= object2.y_loc
        in_horizantal_range = object1.x_loc <= object2.x_loc + object2.size and\
                            object1.x_loc + object1.size >= object2.x_loc
        touch_side = object1.x_loc + object1.size == object2.x_loc or\
                     object2.x_loc + object2.size == object1.x_loc
        touch_top_or_bottom = object1.y_loc + object1.size == object2.y_loc or\
                              object2.y_loc + object2.size == object1.y_loc
        return (in_vertical_range and touch_side) or (in_horizantal_range and touch_top_or_bottom)

    def top(self, objects             )               :
        u"""
        Return the topmost objects (i.e. minimum y_loc). The comparison is done separately for each
        box.
        """
        objects_per_box = self._separate_objects_by_boxes(objects)
        return_set              = set()
        for _, box_objects in list(objects_per_box.items()):
            min_y_loc = min([obj.y_loc for obj in box_objects])
            return_set.update(set([obj for obj in box_objects if obj.y_loc == min_y_loc]))
        return return_set

    def bottom(self, objects             )               :
        u"""
        Return the bottom most objects(i.e. maximum y_loc). The comparison is done separately for
        each box.
        """
        objects_per_box = self._separate_objects_by_boxes(objects)
        return_set              = set()
        for _, box_objects in list(objects_per_box.items()):
            max_y_loc = max([obj.y_loc for obj in box_objects])
            return_set.update(set([obj for obj in box_objects if obj.y_loc == max_y_loc]))
        return return_set

    def above(self, objects             )               :
        u"""
        Returns the set of objects in the same boxes that are above the given objects. That is, if
        the input is a set of two objects, one in each box, we will return a union of the objects
        above the first object in the first box, and those above the second object in the second box.
        """
        objects_per_box = self._separate_objects_by_boxes(objects)
        return_set = set()
        for box in objects_per_box:
            # min_y_loc corresponds to the top-most object.
            min_y_loc = min([obj.y_loc for obj in objects_per_box[box]])
            for candidate_obj in box.objects:
                if candidate_obj.y_loc < min_y_loc:
                    return_set.add(candidate_obj)
        return return_set

    def below(self, objects             )               :
        u"""
        Returns the set of objects in the same boxes that are below the given objects. That is, if
        the input is a set of two objects, one in each box, we will return a union of the objects
        below the first object in the first box, and those below the second object in the second box.
        """
        objects_per_box = self._separate_objects_by_boxes(objects)
        return_set = set()
        for box in objects_per_box:
            # max_y_loc corresponds to the bottom-most object.
            max_y_loc = max([obj.y_loc for obj in objects_per_box[box]])
            for candidate_obj in box.objects:
                if candidate_obj.y_loc > max_y_loc:
                    return_set.add(candidate_obj)
        return return_set

    def _separate_objects_by_boxes(self, objects             )                           :
        u"""
        Given a set of objects, separate them by the boxes they belong to and return a dict.
        """
        objects_per_box                          = defaultdict(list)
        for box in self._boxes:
            for object_ in objects:
                if object_ in box.objects:
                    objects_per_box[box].append(object_)
        return objects_per_box

    @classmethod
    def small(cls, objects             )               :
        return set([obj for obj in objects if obj.size == 10])

    @classmethod
    def medium(cls, objects             )               :
        return set([obj for obj in objects if obj.size == 20])

    @classmethod
    def big(cls, objects             )               :
        return set([obj for obj in objects if obj.size == 30])

    @staticmethod
    def negate_filter(filter_function                                      ,
                      objects             )               :
        # Negate an object filter.
        return objects.difference(filter_function(objects))



# python
#
# See the accompanying LICENSE file.
#
# various automagic documentation updates

import sys


# get the download file names correct

version=sys.argv[1]
url="  <https://github.com/rogerbinns/apsw/releases/download/"+version+"/%s>`__"

pip_template="""
.. pip-begin

Use this (all one command)::

    pip install --user https://github.com/rogerbinns/apsw/releases/download/%s/apsw-%s.zip \\
    --global-option=fetch --global-option=--version --global-option=%s --global-option=--all \\
    --global-option=build --global-option=--enable-all-extensions
""" % (version, version, version.split("-")[0])


download=open("doc/download.rst", "rtU").read()

op=[]
incomment=False
for line in open("doc/download.rst", "rtU"):
    line=line.rstrip()
    if line==".. downloads-begin":
        op.append(line)
        incomment=True
        op.append("")
        op.append("* `apsw-%s.zip" % (version,))
        op.append(url % ("apsw-%s.zip" % version))
        op.append("  (Source, includes this HTML Help)")
        op.append("")
        not64=("2.3", "2.4", "2.5")
        for pyver in ("2.3", "2.4", "2.5", "2.6", "2.7", "3.1", "3.2", "3.3", "3.4", "3.5", "3.6", "3.7"):
            op.append("* Windows Python %s `32bit" % (pyver,))
            op.append(url % ("apsw-%s.win32-py%s.exe" % (version, pyver)))
            if pyver not in not64:
                op.append("  `64bit ")
                op.append(url % ("apsw-%s.win-amd64-py%s.exe" % (version, pyver)))
            op.append("")
        op.append("* `apsw-%s-sigs.zip " % (version,))
        op.append(url % ("apsw-%s-sigs.zip" % version))
        op.append("  GPG signatures for all files")
        op.append("")
        continue
    if line==".. pip-begin":
        op.extend(pip_template.split("\n")[1: ])
        incomment=True
        continue
    if line==".. downloads-end" or line==".. pip-end":
        incomment=False
    if incomment:
        continue
    if line.lstrip().startswith("$ gpg --verify apsw"):
        line=line[:line.index("$")]+"$ gpg --verify apsw-%s.zip.asc" % (version,)
    op.append(line)

op="\n".join(op)
if op!=download:
    open("doc/download.rst", "wt").write(op)

# put usage and description for speedtest into benchmark

import speedtest

benchmark=open("doc/benchmarking.rst", "rtU").read()

op=[]
incomment=False
for line in open("doc/benchmarking.rst", "rtU"):
    line=line.rstrip()
    if line==".. speedtest-begin":
        op.append(line)
        incomment=True
        op.append("")
        op.append(".. code-block:: text")
        op.append("")
        op.append("    $ python speedtest.py --help")
        speedtest.parser.set_usage("Usage: speedtest.py [options]")
        for line in speedtest.parser.format_help().split("\n"):
            op.append("    "+line)
        op.append("")
        op.append("    $ python speedtest.py --tests-detail")
        for line in speedtest.tests_detail.split("\n"):
            op.append("    "+line)
        op.append("")
        continue
    if line==".. speedtest-end":
        incomment=False
    if incomment:
        continue
    op.append(line)

op="\n".join(op)
if op!=benchmark:
    open("doc/benchmarking.rst", "wt").write(op)

# shell stuff

import apsw, StringIO
shell=apsw.Shell()
incomment=False
op=[]
for line in open("doc/shell.rst", "rtU"):
    line=line.rstrip()
    if line==".. help-begin:":
        op.append(line)
        incomment=True
        op.append("")
        op.append(".. code-block:: text")
        op.append("")
        s=StringIO.StringIO()
        def tw(*args):
            return 80
        shell.stderr=s
        shell._terminal_width=tw
        shell.command_help([])
        op.extend(["  "+x for x in s.getvalue().split("\n")])
        op.append("")
        continue
    if line==".. usage-begin:":
        op.append(line)
        incomment=True
        op.append("")
        op.append(".. code-block:: text")
        op.append("")
        op.extend(["  "+x for x in shell.usage().split("\n")])
        op.append("")
        continue
    if line==".. help-end:":
        incomment=False
    if line==".. usage-end:":
        incomment=False
    if incomment:
        continue
    op.append(line)

op="\n".join(op)
if op!=open("doc/shell.rst", "rtU").read():
    open("doc/shell.rst", "wt").write(op)
